
function clik = datastat_condlike(qq, nd, ss, dtype, dparam)
%datastat_condlike    Evaluate conditional likelihoods for multiple classes

numdata = size(ss,2);

switch dtype,
case 1,   % Discrete multinomial
  numfeature = dparam(1);
  dirweight  = dparam(2);

  % likelihood of first data point
  Z    = nd + dirweight*numfeature;
  clik = log(qq(ss(1),:)) - log(Z);

  % likelihood of remaining data points, conditioning on previous
  % if statement is redundant, but faster when numdata=1
  if numdata > 1
    for ii = 1:numdata-1
      qq(ss(ii),:) = qq(ss(ii),:) + 1;
      clik = clik + log(qq(ss(ii+1),:)) - log(Z+ii);
    end
  end

case 2,   % Normal-Inverse-Wishart
  dim   = dparam(1);
  nu    = dparam(2) + nd;
  kappa = dparam(3) + nd;
  numcc = size(qq,2);

  datasum  = qq(1:dim,:);
  datachol = reshape(qq(dim+1:end-1,:), dim, dim, numcc);

  % likelihood of first data point
  clik      = qq(end,:);
  innernorm = 0.5*(nu-dim-1) ./ (kappa.*(kappa+1));
  for cc = 1:numcc
    delmu    = datachol(:,:,cc)' \ (datasum(:,cc) - kappa(cc)*ss(:,1));
    clik(cc) = clik(cc) - innernorm(cc)*sum(delmu.^2,1);
  end

  % likelihood of remaining data points, conditioning on previous
  if numdata > 1
    for cc = 1:numcc
      datachol(:,:,cc) = cholupdate(datachol(:,:,cc), ...
        datasum(:,cc)./sqrt(kappa(cc)));
    end
    for ii = 1:numdata-1
      kappa = kappa + 1;
      nu    = nu + 1;
  
      clik      = clik - 0.5*dim*(log(kappa+1) - log(kappa.*(nu-dim-1)));
      clik      = clik - 0.5*dim*log(2*pi);
      innernorm = 0.5*(nu-dim-1) ./ (kappa.*(kappa+1));
  
      for cc = 1:numcc
        datasum(:,cc)    = datasum(:,cc) + ss(:,ii);
        datachol(:,:,cc) = cholupdate(datachol(:,:,cc), ss(:,ii));
  
        datascov = cholupdate(datachol(:,:,cc), ...
          datasum(:,cc)./sqrt(kappa(cc)), '-');
        delmu    = datascov' \ (datasum(:,cc) - kappa(cc)*ss(:,ii+1));
        clik(cc) = clik(cc) - sum(log(diag(datascov))) - ...
          innernorm(cc)*sum(delmu.^2,1);
      end
    end
  end

case 3,   % Diagonal Normal-Inverse-Wishart
  dim   = dparam(1);
  nu    = dparam(2) + nd;
  kappa = dparam(3) + nd;
  numcc = size(qq,2);

  datasum  = qq(1:dim,:);
  datachol = qq(dim+1:end-1,:);

  % likelihood of first data point
  clik      = qq(end,:);
  innernorm = 0.5*(nu-dim-1) ./ (kappa.*(kappa+1));
  delmu = (datasum - kappa(ones(dim,1),:).*ss(:,ones(1,numcc))) ./ datachol;
  clik  = clik - innernorm.*sum(delmu.^2,1);

  % likelihood of remaining data points, conditioning on previous
  if numdata > 1
    datasq = datachol.^2 + (datasum.^2)./kappa(ones(dim,1),:);

    for ii = 1:numdata-1
      kappa = kappa + 1;
      nu    = nu + 1;
  
      clik      = clik - 0.5*dim*(log(kappa+1) - log(kappa.*(nu-dim-1)));
      clik      = clik - 0.5*dim*log(2*pi);
      innernorm = 0.5*(nu-dim-1) ./ (kappa.*(kappa+1));
  
      datasum  = datasum + ss(:,ii*ones(1,numcc));
      datasq   = datasq  + ss(:,ii*ones(1,numcc)).^2;
      datascov = sqrt(datasq - (datasum.^2)./kappa(ones(dim,1),:));

      delmu = (datasum - kappa(ones(dim,1),:).*ss(:,(ii+1)*ones(1,numcc)))./datascov;
      clik  = clik - sum(log(datascov),1) - innernorm.*sum(delmu.^2,1);
    end
  end

case 4,   % Inverse-Wishart (zero mean)
  dim   = dparam(1);
  nu    = dparam(2) + nd;
  numcc = size(qq,2);

  datachol = reshape(qq(1:end-1,:), dim, dim, numcc);

  % likelihood of first data point
  clik      = qq(end,:);
  innernorm = 0.5*(nu-dim-1);
  for cc = 1:numcc
    delmu    = datachol(:,:,cc)' \ ss(:,1);
    clik(cc) = clik(cc) - innernorm(cc)*sum(delmu.^2,1);
  end

  % likelihood of remaining data points, conditioning on previous
  if numdata > 1
    for ii = 1:numdata-1
      nu = nu + 1;
  
      clik      = clik + 0.5*dim*log(nu-dim-1);
      clik      = clik - 0.5*dim*log(2*pi);
      innernorm = 0.5*(nu-dim-1);
  
      for cc = 1:numcc
        datachol(:,:,cc) = cholupdate(datachol(:,:,cc), ss(:,ii));
  
        delmu    = datachol(:,:,cc)' \ ss(:,ii+1);
        clik(cc) = clik(cc) - sum(log(diag(datachol(:,:,cc)))) - ...
          innernorm(cc)*sum(delmu.^2,1);
      end
    end
  end

case 5,   % Diagonal Inverse-Wishart (zero mean)
  dim   = dparam(1);
  nu    = dparam(2) + nd;
  numcc = size(qq,2);

  datachol  = qq(1:end-1,:);

  % likelihood of first data point
  innernorm = 0.5*(nu-dim-1);
  delmu     = ss(:,ones(1,numcc)) ./ datachol;
  clik      = qq(end,:) - innernorm.*sum(delmu.^2,1);

  % likelihood of remaining data points, conditioning on previous
  if numdata > 1
    datasq = datachol.^2;

    for ii = 1:numdata-1
      nu = nu + 1;
  
      clik      = clik + 0.5*dim*log(nu-dim-1);
      clik      = clik - 0.5*dim*log(2*pi);
      innernorm = 0.5*(nu-dim-1);
      datasq    = datasq  + ss(:,ii*ones(1,numcc)).^2;

      delmu = ss(:,(ii+1)*ones(1,numcc)) ./ sqrt(datasq);
      clik  = clik - 0.5*sum(log(datasq),1) - innernorm.*sum(delmu.^2,1);
    end
  end

end

