
function [qq0,dtype,dparam] = datastat_init(dataprior)
%datastat_init    Initialize cached statistics using given hyperparameters

switch lower(dataprior.type)
case 'discrete',   % Discrete multinomial
  dtype = 1;

  % hyperparameters
  dparam = [dataprior.numfeature; dataprior.dirweight];
  qq0    = dataprior.dirweight*ones(dataprior.numfeature,1);

case 'gaussian',   % Normal-Inverse-Wishart
  dtype = 2;

  % hyperparameters
  nu0    = dataprior.ccovar;
  kappa0 = dataprior.cmean;
  mu0    = dataprior.emean;
  sigma0 = dataprior.ecovar;
  dim    = length(mu0);

  % initialize data statistics
  datasum  = kappa0*mu0;
  datachol = chol(nu0*sigma0);
  %datachol = cholupdate(datachol, sqrt(kappa0)*mu0);

  % store logdet normalization for faster likelihood evaluation
  covnorm = (kappa0+1)/(kappa0*(nu0-dim-1));
  logdet  = -0.5*dim*log(covnorm) - sum(log(diag(datachol)));
  logdet  = logdet - 0.5*dim*log(2*pi);

  qq0    = [datasum; datachol(:); logdet];
  dparam = [dim; nu0; kappa0];

case 'diagonalgaussian',   % Diagonal Normal-Inverse-Wishart
  dtype = 3;

  % hyperparameters
  nu0    = dataprior.ccovar;
  kappa0 = dataprior.cmean;
  mu0    = dataprior.emean;
  sigma0 = dataprior.ecovar;
  dim    = length(mu0);

  % handle expected covariance in matrix or vector form
  if prod(size(sigma0)) > length(sigma0)
    sigma0 = diag(sigma0);
  end

  % initialize data statistics
  datasum  = kappa0*mu0;
  datachol = sqrt(nu0*sigma0);

  % store logdet normalization for faster likelihood evaluation
  covnorm = (kappa0+1)/(kappa0*(nu0-dim-1));
  logdet  = -0.5*dim*log(covnorm) - sum(log(datachol));
  logdet  = logdet - 0.5*dim*log(2*pi);

  qq0    = [datasum; datachol; logdet];
  dparam = [dim; nu0; kappa0];

case 'zeromeangaussian',   % Inverse-Wishart (zero mean)
  dtype = 4;

  % hyperparameters
  nu0    = dataprior.ccovar;
  sigma0 = dataprior.ecovar;
  dim    = size(sigma0,1);

  % initialize data statistics
  datachol = chol(nu0*sigma0);

  % store logdet normalization for faster likelihood evaluation
  logdet  = 0.5*dim*log(nu0-dim-1) - sum(log(diag(datachol)));
  logdet  = logdet - 0.5*dim*log(2*pi);

  qq0    = [datachol(:); logdet];
  dparam = [dim; nu0];

case 'zeromeandiagonalgaussian',   % Diagonal Inverse-Wishart (zero mean)
  dtype = 5;

  % hyperparameters
  nu0    = dataprior.ccovar;
  sigma0 = dataprior.ecovar;

  % handle expected covariance in matrix or vector form
  if prod(size(sigma0)) > length(sigma0)
    sigma0 = diag(sigma0);
  end
  dim = length(sigma0);

  % initialize data statistics
  datachol = sqrt(nu0*sigma0);

  % store logdet normalization for faster likelihood evaluation
  logdet  = 0.5*dim*log(nu0-dim-1) - sum(log(datachol));
  logdet  = logdet - 0.5*dim*log(2*pi);

  qq0    = [datachol; logdet];
  dparam = [dim; nu0];

end
 
