
% Visualize parsing of test scenes using transformed DP models

clear all

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% DEMO PARAMETERS
showOffices   = 1        % Set to 0 for street scenes, 1 for office scenes
showParts     = 1        % Set to 1 for multiple-part TDP, 0 for single-part
showImageTest = 15       % Index of test image to visualize 
ROOTHOME = 'database';   % Folder that has the databases of images
DEMOHOME = 'demoModels'; % Folder that has the trained TDP models
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% SCENE & PLOT PARAMETERS
if ~showOffices
  if showParts
    demofile = fullfile(DEMOHOME, 'streetPartTDP.mat');
  else
    demofile = fullfile(DEMOHOME, 'streetBlobTDP.mat');
  end
  load(demofile,'TDP','imagedata','numtrain','numgibbsTrain','testgroup');
  HOMEIMAGES = fullfile(ROOTHOME, 'streetFixScale', 'Images');

  plotnames  = {'background', 'car', 'building', 'road', 'tree'};
  plotorder  = [0 2 3 4 1];
  plotcolor  = [0.5 0.5 0.5; 1 0 0; 1 0 1; 0 0 1; 0 1 0];
  plotstyle  = {'k', 'r', 'm', 'b', 'g'};
  plotdetail = 1;
  plottable  = [1:4];
else
  if showParts
    demofile = fullfile(DEMOHOME, 'officePartTDP.mat');
  else
    demofile = fullfile(DEMOHOME, 'officeBlobTDP.mat');
  end
  load(demofile,'TDP','imagedata','numtrain','numgibbsTrain','testgroup');
  HOMEIMAGES = fullfile(ROOTHOME, 'office', 'Images');

  plotnames  = {'background', 'screen', 'mouse', 'keyboard', 'background'};
  plotorder  = [0 4 1 3 2];
  plotcolor  = [0.5 0.5 0.5; 1 0 0; 0 0 1; 0 1 0; 0.5 0.5 0.5];
  plotstyle  = {'k', 'r', 'b', 'g', 'k'};
  plotdetail = 1;
  plottable  = [1:3];
end
plotcolortable = [1 1 0; 1 .5 0; 0 1 .5; 0 1 1; .5 .5 .5; 1 0 .5; 1 .5 0; ...
  1 0 1; 1 .25 .5; 0 .5 1; 1 .8 .5; .8 1 .8; .7 1 .6];
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

trainingIterations = sum(numgibbsTrain); 
numcat = length(plotorder);
dim = 2;

% Read selected image (from the test set)
testimage = testgroup(showImageTest);
rawimage  = imread(fullfile(HOMEIMAGES, imagedata.files{testimage}));

% Create figure
screenSize = get(0, 'ScreenSize'); screenSize(1) = 100; screenSize(2) = 100; screenSize(3:4) = screenSize(3:4)-200;
Hfig = gcf;
set (Hfig, 'position', screenSize);

% Run 100 iterations on test image:
for iter = 1:100
  TDP = doGibbs(TDP, 1, 5, {}, testimage, 0, [], true);
  j = 1;

  % transformed clusters for displayed images
  [tableObj,tableProb,tableTran,tableAssign,partAssign] = gettables(TDP, testimage);
  [partParam,partProb,objProb,objTran,objLabel] = getobjects(TDP);
  numobj   = length(objProb);   % number of global object clusters
  objLabel = max(objLabel,0);   % ignore negative labels

  % extract mean & covariance of parts for each object
  muPart  = cell(1,numobj);
  sigPart = cell(1,numobj);
  
  for cc = 1:numobj
    numpart(cc) = length(partProb{cc});
    muPart{cc}  = partParam{cc}{1}(1:dim,:);
  
    if size(partParam{cc}{1},1) > 2*dim
      sigPart{cc} = reshape(partParam{cc}{1}(dim+1:end,:),dim,dim,numpart(cc));
    else
      sigPart{cc} = zeros(dim,dim,numpart(cc));
      for kk = 1:numpart(cc)
        sigPart{cc}(:,:,kk) = diag(partParam{cc}{1}(dim+1:end,kk));
      end
    end
  end

  % look for instances that are assigned to at least 5 features 
  numfeat = length(tableAssign{j});
  plotind = find(tableProb{j} >= 5/numfeat);

% 1) show image with transformed object names overlaid
  clf; sp = 0; Nx = 4; Ny = 2;
  Hax = axes('position', [mod(sp,Nx)/Nx (Ny-fix(sp/Nx)-1)/Ny 1/Nx 1/Ny], ...
    'parent', Hfig); sp = sp+1;
  image(rawimage,'parent',Hax); axis(Hax,'image'); axis(Hax,'off');
  title({'Test image and detection result', sprintf('Gibbs iteration = %d', iter)})

  % plot names of each object instance
  for tt = plotind
    cc = tableObj{j}(tt);

    if (isempty(plottable) || ismember(objLabel(cc),plottable)) 
      muT = muPart{cc}*partProb{cc}' + tableTran{j}(:,tt);
      text(muT(1), muT(2), plotnames{objLabel(cc)+1}, ...
        'horizontalalignment', 'center', 'verticalalignment', 'middle', ...
        'fontweight', 'bold', 'color', plotcolor(objLabel(cc)+1,:));
    end
  end

% 2) show segmentation based on current assignments
  featObj    = zeros(1,numfeat);
  assignind  = find(tableAssign{j} > 0);
  featObj(assignind) = objLabel(tableObj{j}(tableAssign{j}(assignind)));
  featWeight = ones(1,numfeat);

  % change order in which features displayed
  newind = [];
  for oo = 1:numcat
    newind = [newind find(featObj == plotorder(oo))];
  end

  Hax = axes('position', [mod(sp,Nx)/Nx (Ny-fix(sp/Nx)-1)/Ny 1/Nx 1/Ny], ...
    'parent', Hfig); sp = sp+1; 
  displayROI(rawimage, imagedata.shapeROI_left{testimage}(:,newind), ...
    featObj(newind)+1, plotcolor, featWeight(newind), Hax); 
  title(Hax,'Feature segmentation');
  
% 3) show transformed tables
  muT = []; sigmaT = []; weightsT = []; colorsT = {};
  for tt = plotind
    cc = tableObj{j}(tt);

    if (isempty(plottable) || ismember(objLabel(cc),plottable)) 
      muT      = cat(2, muT, muPart{cc}+tableTran{j}(:,tt*ones(1,numpart(cc))));
      sigmaT   = cat(3, sigmaT, sigPart{cc});
      weightsT = cat(2, weightsT, tableProb{j}(tt)*partProb{cc});
      colorsT  = cat(2, colorsT, repmat(plotstyle(objLabel(cc)+1),1,numpart(cc)));
    end
  end

  Hax = axes('position', [mod(sp,Nx)/Nx (Ny-fix(sp/Nx)-1)/Ny 1/Nx 1/Ny], ...
    'parent', Hfig); sp = sp+1; 
  image(rawimage,'parent',Hax); axis(Hax,'image'); axis(Hax,'off');
  plotGaussMix(muT, sigmaT, weightsT, colorsT, Hax);
  title(Hax,'Transformed global clusters');

% 4) show instances corresponding to specified detail class
  selectTable = plotind(find(objLabel(tableObj{j}(plotind)) == plotdetail));

  dataind  = []; datalabel = [];
  partbase = 0; 
  for t = 1:length(selectTable)
    tt = selectTable(t);

    newdata   = find(tableAssign{j} == tt);
    dataind   = [dataind, newdata];
    datalabel = [datalabel, partbase + partAssign{j}(newdata)];
    partbase  = partbase + numpart(tableObj{j}(tt));
  end

  Hax = axes('position', [mod(sp,Nx)/Nx (Ny-fix(sp/Nx)-1)/Ny 1/Nx 1/Ny], ...
    'parent', Hfig); sp = sp+1; 
  displayROI(rawimage, imagedata.shapeROI_left{testimage}(:,dataind), ...
    datalabel, plotcolortable, [], Hax); title(Hax,'');
  title(Hax,{'Segmented instances/parts of', ...
    ['transformed ' plotnames{plotdetail+1} ' clusters']})

  % Display comments and general information about training:
  axes('position', [0 0 1 1/Ny]); axis('off')
  text(0.2, .9, {...
    sprintf('Training was performed on %d images. ', numtrain), ...
    sprintf('Gibbs sampling on the training set was run for %d iterations.', trainingIterations), ...
    '', ...
    'Here we show how test image segmentations evolve as the Gibbs sampler runs, for a fixed set of training image assignments.', ...
    'Note that clusters can move, appear/disappear, and can flip identity from one iteration to the next.', ...
    'In many images the sampler begins to stabilize after 30 iterations.', ...
  }, 'verticalalignment', 'top');
  drawnow;
end

fprintf('Finished %d Gibbs sampling iterations.\n',iter);

