
function [objpart,catprob] = ...
  doGibbs(objpart, numiter, visparam, sampgroup, testgroup, testiter, nonRB)
%doGibbs     Approximates posterior distributions via Gibbs sampling
%
%    [OP,catprob] = doGibbs(OP,numiter,visparam,sampgroup,testgroup,testiter,nonRB)
%
%  Learning and inference via Gibbs sampling, where
%    numiter    = number of iterations (one iteration resamples all 
%                 assignments of data to classes, in all chosen groups, once)
%    visparam   = cell array of arguments to visualize, or empty for
%                 no plot of learning process
%    sampgroup  = groups on which to run Gibbs sampling, typically the
%                 groups corresponding to training data (by default, all)
%    testgroup  = groups to estimate posterior category probabilites at
%                 each iteration, typically the groups corresponding to
%                 test data (by default, none)
%    testiter   = iterations at which to compute and record posterior category
%                 probabilities (by default, testiter = [1:numiter])
%    nonRB      = flag to not use Rao-Blackwellized sampling, and explicitly
%                 resample parameters and weights (by default, false)
%    catprob    = posterior log-likelihoods logP(group|object) for testgroup, 
%                 size(catprob) = [numobj length(testgroup) length(testiter)]

  % default parameters
  if nargin < 2
    numiter = 1;
  end
  if nargin < 3
    visparam = [];
  end
  if nargin < 4
    sampgroup = [1:objpart.numgroup];
  end
  if nargin < 5
    testgroup = [];
  end
  if nargin < 6
    testiter = [1:numiter];
  end
  if nargin < 7
    nonRB = 0;
  end

  % check for test images in given training groups
  trainind = find(objpart.category(sampgroup) > 0);
  if length(trainind) < length(sampgroup)
    warning('Test images in given training groups; ignoring...');
    sampgroup = sampgroup(trainind);
  end

  % store inferred test categories after each iteration
  numtest   = length(testgroup);
  catprob   = zeros(objpart.numobj, numtest, length(testiter));
  testindex = 0;

  for iter = 1:numiter
    if (nonRB > 0)
      objpart = gibbs_nonRB(objpart, sampgroup);
    else
      objpart = gibbs_noref(objpart, sampgroup);
    end

    if ~isempty(visparam)
      visualize(objpart, visparam{:});
      drawnow;
    end

    if (numtest > 0) && (sum(iter == testiter) > 0)
      testindex = testindex + 1;
      catprob(:,:,testindex) = estcategory(objpart, testgroup); 
    end

    fprintf(1,'%d.',iter);
  end
  fprintf(1,'\n');

