
function objpart = gibbs_nonRB(objpart, groups)
%gibbs_nonRB     Iteration of non-Rao-Blackwellised Gibbs sampler, without reference 

% determine number of datatypes used in data likelihoods
% hardcoded for 1 or 2 datatypes to avoid overhead of loop over datatypes
if objpart.numtype > 2
  error('Gibbs sampler supports at most two data types');
end
usetypeB = (objpart.numtype > 1);

% static parameters
alpha    = objpart.alpha;
dtype    = objpart.dtype;
dparam   = objpart.dparam{1};

% class (part) level state variables
classnd = objpart.classnd;
classqq = objpart.classqq{1};

% parameters for second datatype
if usetypeB
  dparamB  = objpart.dparam{2};
  classqqB = objpart.classqq{2};
end

% sample a current parameter value for each cluster
numclass = objpart.numclass;
for cc = 1:numclass
  classpp(:,cc) = datastat_sampparam(classqq(:,cc), classnd(cc), ...
    1, dtype(1), dparam);
  if usetypeB
    classppB(:,cc) = datastat_sampparam(classqqB(:,cc), classnd(cc), ...
      1, dtype(2), dparamB);
  end
end

% sample current mixture weights
numobj = objpart.numobj;
objnd  = objpart.objnd;
for oo = 1:numobj
  objprob(oo,:) = dirichlet_rnd(objnd(oo,:)+alpha, 1)';
end

% resample groups in random order
for j = randperm(length(groups))
  jj = groups(j);

  % state variables for current group
  numdata  = objpart.numdata(jj);
  datass   = objpart.datass{jj}{1};
  datacc   = objpart.datacc{jj};
  category = objpart.category(jj);

  % state variables for second datatype
  if usetypeB
    datassB = objpart.datass{jj}{2};
  else
    datassB = zeros(0,numdata);
  end

  % sample part associated with each data item, in random order
  for ii = randperm(numdata)
    ss    = datass(:,ii);
    ssB   = datassB(:,ii);
    oldcc = datacc(ii);

    % compute conditional likelihood factors for each class
    clik = dataparam_indeplike(classpp, ss, dtype(1), dparam);
    if usetypeB
      clik = clik + dataparam_indeplike(classppB, ssB, dtype(2), dparamB);
    end

    % sample a new class assignment, accounting for Dirichlet prior
    clik  = exp(clik-max(clik)) .* objprob(category,:);
    newcc = multinomial_rnd(clik, 1);

    % update data statistics if necessary
    if newcc ~= oldcc
      if oldcc > 0
        classqq(:,oldcc) = datastat_del(classqq(:,oldcc), ...
          classnd(oldcc), ss, dtype(1), dparam);
        if usetypeB
          classqqB(:,oldcc) = datastat_del(classqqB(:,oldcc), ...
            classnd(oldcc), ssB, dtype(2), dparamB);
        end
        classnd(oldcc)        = classnd(oldcc) - 1;
        objnd(category,oldcc) = objnd(category,oldcc) - 1;
      end

      datacc(ii)       = newcc;
      classqq(:,newcc) = datastat_add(classqq(:,newcc), ...
        classnd(newcc), ss, dtype(1), dparam);
      if usetypeB
        classqqB(:,newcc) = datastat_add(classqqB(:,newcc), ...
          classnd(newcc), ssB, dtype(2), dparamB);
      end
      classnd(newcc)        = classnd(newcc) + 1;
      objnd(category,newcc) = objnd(category,newcc) + 1;
    end
  end

  objpart.datacc{jj} = datacc;
end

% store class state variables for next iteration
objpart.objnd = objnd;
objpart.classqq{1} = classqq;
objpart.classnd = classnd;
if usetypeB
  objpart.classqq{2} = classqqB;
end

