
function [sceneobj,catprob] = ...
  doGibbs(sceneobj,numiter,visparam,sampgroup,testgroup,numEM,testiter)
%doGibbs     Approximates posterior distributions via Gibbs sampling
%
%    [SO,catprob] = doGibbs(SO,numiter,visparam,sampgroup,testgroup,numEM,testiter)
%
%  Learning and inference via Gibbs sampling, where
%    numiter   = number of iterations (one iteration resamples all 
%                assignments of data to classes, in all chosen groups, once)
%    visparam  = cell array of arguments to visualize, or empty for
%                no plot of learning process
%    sampgroup = groups on which to run Gibbs sampling, typically the
%                groups corresponding to training data, and also the test
%                groups if reference positions are used (by default, all)
%    testgroup = groups to estimate posterior object category probabilites 
%                for each feature at each iteration, typically the groups 
%                corresponding to test data (by default, none)
%    numEM     = number of reference position incremental EM updates
%                between each resampled feature assignment, or 0 to 
%                Gibbs sample reference positions instead (default)
%    testiter  = iterations at which to compute and record posterior category
%                probabilities (by default, testiter = [1:numiter])
%    catprob   = posterior log-likelihoods logP(obj|data,scene) for testgroup, 
%                where catprob = length(testgroup)-dim. cell array and
%                size(catprob{j}) = [numdata(testgroup(j)) numobj length(testiter)]

  % Default parameters
  if nargin < 2
    numiter = 1;
  end
  if nargin < 3
    visparam = [];
  end
  if nargin < 4
    sampgroup = [1:sceneobj.numgroup];
  end
  if nargin < 5
    testgroup = [];
  end
  if nargin < 6
    numEM = 0;
  end
  if nargin < 7
    testiter = [1:numiter];
  end

  % store inferred test categories after each iteration
  numobj    = sceneobj.numobj;
  numdata   = sceneobj.numdata;
  numtest   = length(testgroup);
  testindex = 0;

  catprob = cell(numtest,1);
  for j = 1:numtest
    catprob{j} = zeros(numdata(testgroup(j)), numobj, length(testiter));
  end

  for iter = 1:numiter
    if sceneobj.useref
      if numEM > 0
        sceneobj = gibbs_refEM(sceneobj, sampgroup, numEM);
      else
        sceneobj = gibbs_refSamp(sceneobj, sampgroup);
      end
    else
      sceneobj = gibbs_noref(sceneobj, sampgroup);
    end

    if ~isempty(visparam)
      visualize(sceneobj, visparam{:});
      drawnow;
    end

    if (numtest > 0) && (sum(iter == testiter) > 0)
      testindex = testindex + 1;
      for j = 1:numtest
        objclassprob         = estassign(sceneobj, testgroup(j));
        catprob{j}(:,:,testindex) = sum(objclassprob,3);
      end
    end

    fprintf(1,'%d.',iter);
  end
  fprintf(1,'\n');

