
function posterior = estassign(sceneobj, group, scene)
%estassign    Posterior category/class assignment probabilities for features 
%
%    post = estassign(SO, group, scene)
%
%  Joint posterior probability that each (object category, class) pair
%  generated each feature, where
%    group = single group for which data assignment probabilities computed
%    scene = assumed scene type under which to compute probabilities
%            (defaults to scene specified when data inserted)
%    post  = (numdata x numobj x numclass) matrix of P(obj,class|data,scene)

  % default parameters
  if nargin < 3 || scene == 0
    scene = sceneobj.category(group);
  end
  if scene == 0
    error('Need scene type to compute object/class assignment probabilities');
  end

  if length(group) > 1
    error('Only estimates object/class assignments for a single group');
  end

  % static parameters
  gamma    = sceneobj.gamma;
  alpha    = sceneobj.alpha;
  numobj   = sceneobj.numobj;
  numclass = sceneobj.numclass;
  numtype  = sceneobj.numtype;
  dtype    = sceneobj.dtype;
  dparam   = sceneobj.dparam;
  useref   = sceneobj.useref;

  % class (part) level state variables
  classnd = sceneobj.classnd;
  classqq = sceneobj.classqq;

  % data for chosen group
  numdata = sceneobj.numdata(group);
  datass  = sceneobj.datass{group};

  % compute P(object|scene)
  objprob = sceneobj.scenend(:,scene) + gamma;
  objprob = objprob / sum(objprob);

  % compute P(class|object)
  classprob = sceneobj.objnd + alpha;
  classnorm = sum(classprob,2);
  classprob = classprob ./ repmat(classnorm,1,numclass);

  % compute logP(feature|class,object), using current reference estimate
  clik = zeros(numdata, numobj, numclass);

  if useref
    startind = 2;

    dim   = dparam{1}(1);
    theta = datastat_estparam(classqq{1}, classnd, dtype(1), dparam{1});

    classmean  = theta(1:dim,:);
    classcovar = reshape(theta(dim+1:end,:), dim, dim, numclass);
    groupmean  = sceneobj.groupmean(:,group);
    groupcovar = sceneobj.groupcovar(:,:,group);

    for oo = 1:numobj
      indA = dim*(oo-1) + 1;
      indB = dim*oo;

      for cc = 1:numclass
        localmean     = groupmean(indA:indB) + classmean(:,cc);
        localcovar    = groupcovar(indA:indB,indA:indB) + classcovar(:,:,cc);
        clik(:,oo,cc) = -0.5*log(det(localcovar));

        for ii = 1:numdata
          delss = datass{1}(:,ii) - localmean;
          clik(ii,oo,cc) = clik(ii,oo,cc) - 0.5* delss' * (localcovar \ delss);
        end
      end
    end
  else
    startind = 1;
  end

  for dd = startind:numtype
    classqqLocal = classqq{dd};
    datassLocal  = datass{dd};
    dtypeLocal   = dtype(dd);
    dparamLocal  = dparam{dd};

    for ii = 1:numdata
      datallik = datastat_condlike(classqqLocal, classnd, datassLocal(:,ii), ...
        dtypeLocal, dparamLocal);
      clik(ii,:,:) = clik(ii,:,:) + ...
        repmat(reshape(datallik, [1 1 numclass]), [1 numobj 1]);
    end
  end

  % compute P(class,object|feature,scene)
  posterior = exp(clik) .* ...
    repmat(reshape(classprob, [1 numobj numclass]), [numdata 1 1]);
  posterior = posterior .* repmat(objprob', [numdata 1 numclass]);
  probnorm  = sum(sum(posterior,3),2);
  posterior = posterior ./ repmat(probnorm, [1 numobj numclass]);

