
function sceneobj = gibbs_noref(sceneobj, groups)
%gibbs_noref     Iteration of Gibbs sampler, neglecting reference positions

% determine number of datatypes used in data likelihoods
% hardcoded for 1 or 2 datatypes to avoid overhead of loop over datatypes
if sceneobj.numtype > 2
  error('Gibbs sampler supports at most two data types');
end
usetypeB = (sceneobj.numtype > 1);

% static parameters
alpha    = sceneobj.alpha;
gamma    = sceneobj.gamma;
dtype    = sceneobj.dtype;
dparam   = sceneobj.dparam{1};
numobj   = sceneobj.numobj;
numclass = sceneobj.numclass;

% class (part) level state variables
classnd = sceneobj.classnd;
classqq = sceneobj.classqq{1};
objnd   = sceneobj.objnd;

% parameters for second datatype
if usetypeB
  dparamB  = sceneobj.dparam{2};
  classqqB = sceneobj.classqq{2};
end

% resample groups in random order
for j = randperm(length(groups))
  jj = groups(j);

  % state variables for current group
  numdata  = sceneobj.numdata(jj);
  datass   = sceneobj.datass{jj}{1};
  datacc   = sceneobj.datacc{jj};
  dataoo   = sceneobj.dataoo{jj};
  labeloo  = sceneobj.labeloo{jj};
  category = sceneobj.category(jj);
  scenend  = sceneobj.scenend(:,category);

  % state variables for second datatype
  if usetypeB
    datassB = sceneobj.datass{jj}{2};
  else
    datassB = zeros(0,numdata);
  end

  % sample part associated with each data item, in random order
  for ii = randperm(numdata)
    ss    = datass(:,ii);
    ssB   = datassB(:,ii);
    oldcc = datacc(ii);
    oldoo = dataoo(ii);

    % temporarily remove data item from model, unless not yet assigned
    if oldcc > 0
      classqq(:,oldcc) = datastat_del(classqq(:,oldcc), ...
        classnd(oldcc), ss, dtype(1), dparam);
      if usetypeB
        classqqB(:,oldcc) = datastat_del(classqqB(:,oldcc), ...
          classnd(oldcc), ssB, dtype(2), dparamB);
      end

      classnd(oldcc)     = classnd(oldcc) - 1;
      objnd(oldoo,oldcc) = objnd(oldoo,oldcc) - 1;
      scenend(oldoo)     = scenend(oldoo) - 1;
    end

    % compute conditional likelihood factors for each class
    clik = datastat_condlike(classqq, classnd, ss, dtype(1), dparam);
    if usetypeB
      clik = clik + datastat_condlike(classqqB, classnd, ssB, dtype(2), dparamB);
    end

    % if object unlabelled, compute probability of every object-part pair
    % otherwise, compute probability of each part given known object
    % account for scene and object-level Dirichlet priors
    if labeloo(ii) > 0
      newoo = labeloo(ii);
      clik  = exp(clik-max(clik)) .* (objnd(newoo,:) + alpha);
      newcc = multinomial_rnd(clik, 1);
    else
      clik = exp(clik-max(clik));

      prob     = objnd + alpha;
      probnorm = (scenend+gamma) ./ sum(prob,2);
      prob     = prob .* clik(ones(numobj,1),:) .* probnorm(:,ones(1,numclass));

      % allow exclusion of single object category
      if labeloo(ii) < 0
        prob(-labeloo(ii),:) = 0;
      end

      newoocc = multinomial_rnd(prob(:),1);
      newcc   = fix((newoocc-1)/numobj) + 1;
      newoo   = newoocc - (newcc-1)*numobj;
    end

    % add data item back to model
    datacc(ii) = newcc;
    dataoo(ii) = newoo;

    classqq(:,newcc) = datastat_add(classqq(:,newcc), ...
      classnd(newcc), ss, dtype(1), dparam);
    if usetypeB
      classqqB(:,newcc) = datastat_add(classqqB(:,newcc), ...
        classnd(newcc), ssB, dtype(2), dparamB);
    end

    classnd(newcc)     = classnd(newcc) + 1;
    objnd(newoo,newcc) = objnd(newoo,newcc) + 1;
    scenend(newoo)     = scenend(newoo) + 1;
  end

  sceneobj.datacc{jj}          = datacc;
  sceneobj.dataoo{jj}          = dataoo;
  sceneobj.scenend(:,category) = scenend;
end

% store class state variables for next iteration
sceneobj.classqq{1} = classqq;
sceneobj.classnd    = classnd;
sceneobj.objnd      = objnd; 
if usetypeB
  sceneobj.classqq{2} = classqqB;
end

