
function sceneobj = gibbs_refEM(sceneobj, groups, numEM)
%gibbs_refEM     Iteration of Gibbs sampler, using EM for reference positions

% determine number of datatypes used in data likelihoods
% hardcoded for 1 or 2 datatypes to avoid overhead of loop over datatypes
if sceneobj.numtype > 2
  error('Gibbs sampler supports at most two data types');
end
usetypeB = (sceneobj.numtype > 1);

% verify that first datatype is Gaussian, to match reference position code
if sceneobj.dtype(1) ~= 2
  error('Reference position EM updates assume first datatype is Gaussian');
end

% static parameters
alpha    = sceneobj.alpha;
gamma    = sceneobj.gamma;
dtype    = sceneobj.dtype;
dparam   = sceneobj.dparam{1};
numobj   = sceneobj.numobj;
numclass = sceneobj.numclass;

% reference position parameters
dim      = sceneobj.rparam(1);

% class position parameters
classnu    = dparam(2);
classkappa = dparam(3);

% class (part) level state variables
classnd   = sceneobj.classnd;
objnd     = sceneobj.objnd;

% convert Gaussian class statistics to form required by EM updates
classsum  = sceneobj.classqq{1}(1:dim,:);
classout  = reshape(sceneobj.classqq{1}(dim+1:end-1,:), dim, dim, numclass);

for cc = 1:numclass
  classout(:,:,cc) = cholupdate(classout(:,:,cc), ...
    classsum(:,cc)./sqrt(classkappa+classnd(cc)));
  classout(:,:,cc) = classout(:,:,cc)'*classout(:,:,cc);
end

% parameters for second datatype
if usetypeB
  dparamB  = sceneobj.dparam{2};
  classqqB = sceneobj.classqq{2};
end

% temporary variables used in EM updates
classprecis = zeros(dim,dim,numclass);
classcovar  = zeros(dim,dim,numclass);
classmean   = zeros(dim,numclass);

% resample groups in random order
for j = randperm(length(groups))
  jj = groups(j);

  % state variables for current group
  numdata  = sceneobj.numdata(jj);
  datass   = sceneobj.datass{jj}{1};
  datacc   = sceneobj.datacc{jj};
  dataoo   = sceneobj.dataoo{jj};
  labeloo  = sceneobj.labeloo{jj};
  category = sceneobj.category(jj);
  scenend  = sceneobj.scenend(:,category);
  refnu    = sceneobj.rparam(2) + sceneobj.sceneng(category);
  refkappa = sceneobj.rparam(3) + sceneobj.sceneng(category);

  % state variables for second datatype
  if usetypeB
    datassB = sceneobj.datass{jj}{2};
  else
    datassB = zeros(0,numdata);
  end

  % group (image) state variables for reference position estimation
  groupnd    = sceneobj.groupnd(:,:,jj);
  groupsum   = sceneobj.groupsum(:,:,:,jj);
  groupmean  = sceneobj.groupmean(:,jj);
  groupcovar = sceneobj.groupcovar(:,:,jj);

  % reference position state variables 
  refsum     = sceneobj.refsum(:,category);
  refchol    = sceneobj.refchol(:,:,category);

  % compute starting reference position mean and covariance
  refmean  = refsum / refkappa;
  refcovar = cholupdate(refchol, refsum / sqrt(refkappa), '-');
  refcovar = refcovar'*refcovar * ((refkappa+1)/(refkappa*(refnu+numobj*dim+1)));

  % remove current group from reference position statistics
  refsum  = refsum - groupmean;
  refchol = cholupdate(refchol, groupmean, '-');
  refchol = chol(refchol'*refchol - groupcovar);

  % sample part associated with each data item, in random order
  for ii = randperm(numdata)
    ss    = datass(:,ii);
    ssB   = datassB(:,ii);
    oldcc = datacc(ii);
    oldoo = dataoo(ii);

    % temporarily remove data item from model, unless not yet assigned
    if oldcc > 0
      oldref = groupmean(dim*(oldoo-1)+1:dim*oldoo);
      oldcov = groupcovar(dim*(oldoo-1)+1:dim*oldoo,dim*(oldoo-1)+1:dim*oldoo);
      classsum(:,oldcc)    = classsum(:,oldcc) - (ss - oldref);
      classout(:,:,oldcc)  = classout(:,:,oldcc) - oldcov - (ss-oldref)*(ss-oldref)';
      groupsum(:,oldoo,oldcc) = groupsum(:,oldoo,oldcc) - ss;

      if usetypeB
        classqqB(:,oldcc) = datastat_del(classqqB(:,oldcc), ...
          classnd(oldcc), ssB, dtype(2), dparamB);
      end

      classnd(oldcc)       = classnd(oldcc) - 1;
      objnd(oldoo,oldcc)   = objnd(oldoo,oldcc) - 1;
      scenend(oldoo)       = scenend(oldoo) - 1;
      groupnd(oldoo,oldcc) = groupnd(oldoo,oldcc) - 1;
    end

    % incremental EM updates of reference position mean and covariance
    classnorm = (classkappa+classnd) ./ (classkappa+classnd+1);
    classnorm = classnorm .* (classnu+classnd+dim+1);

    for e = 1:numEM
      % store previous group statistics for later class M-step
      oldmean  = groupmean;
      oldcovar = groupcovar;

      % E-step: Gaussian posterior over reference positions for current image
      for cc = 1:numclass
        classmean(:,cc)     = classsum(:,cc) ./ (classkappa+classnd(cc));
        classprecis(:,:,cc) = classnorm(cc) * inv(classout(:,:,cc) - ...
          classsum(:,cc)*classsum(:,cc)' ./ (classkappa+classnd(cc)));
      end

      groupmean  = zeros(numobj*dim,1);
      groupcovar = inv(refcovar);

      for oo = 1:numobj
        indA = dim*(oo-1) + 1;
        indB = dim*oo;

        for cc = 1:numclass
          groupcovar(indA:indB,indA:indB) = groupcovar(indA:indB,indA:indB) + ...
            groupnd(oo,cc)*classprecis(:,:,cc);
          groupmean(indA:indB) = groupmean(indA:indB) + classprecis(:,:,cc) * ...
            (groupsum(:,oo,cc) - groupnd(oo,cc)*(classmean(:,cc)+refmean(indA:indB)));
        end
      end

      groupcovar = inv(groupcovar);
      groupmean  = refmean + groupcovar*groupmean;

      % M-step: Reference position mean and covariance (current scene only)
      refmean  = (refsum + groupmean) / refkappa;
      refnorm  = (refkappa+1) / (refkappa*(refnu+numobj*dim+1));
      refcovar = cholupdate(refchol, groupmean);
      refcovar = cholupdate(refcovar, sqrt(refkappa)*refmean, '-');
      refcovar = refnorm*(groupcovar + refcovar'*refcovar);
      
      % M-step: Class position statistics (all classes)
      delmean = groupmean - oldmean;
      for oo = 1:numobj
        indA = dim*(oo-1) + 1;
        indB = dim*oo;

        for cc = 1:numclass
          classsum(:,cc)   = classsum(:,cc) - groupnd(oo,cc)*delmean(indA:indB);
          classout(:,:,cc) = classout(:,:,cc) + groupnd(oo,cc) * ...
            (groupcovar(indA:indB,indA:indB) - oldcovar(indA:indB,indA:indB));
          classout(:,:,cc) = classout(:,:,cc)-groupsum(:,oo,cc)*delmean(indA:indB)';
          classout(:,:,cc) = classout(:,:,cc)-delmean(indA:indB)*groupsum(:,oo,cc)';
          classout(:,:,cc) = classout(:,:,cc) + groupnd(oo,cc) * ...
            (groupmean(indA:indB)*groupmean(indA:indB)' - ...
             oldmean(indA:indB)*oldmean(indA:indB)');
        end
      end
    end

    % compute conditional likelihood factors for each class 
    if usetypeB
      clik = datastat_condlike(classqqB, classnd, ssB, dtype(2), dparamB);
    else
      clik = zeros(1,numclass);
    end

    for cc = 1:numclass
      classmean(:,cc)    = classsum(:,cc) ./ (classkappa+classnd(cc));
      classcovar(:,:,cc) = inv(classnorm(cc)) * (classout(:,:,cc) - ...
        (classsum(:,cc)*classsum(:,cc)') ./ (classkappa+classnd(cc)));
    end

    % if object unlabelled, compute probability of every object-part pair
    % otherwise, compute probability of each part given known object
    % account for scene and object-level Dirichlet priors
    if labeloo(ii) > 0
      newoo = labeloo(ii);
      indA  = dim*(newoo-1) + 1;
      indB  = dim*newoo;

      for cc = 1:numclass
        localmean  = groupmean(indA:indB) + classmean(:,cc);
        localcovar = groupcovar(indA:indB,indA:indB) + classcovar(:,:,cc);
        clik(cc)   = clik(cc) - 0.5*log(det(localcovar)) - ...
          0.5*(ss-localmean)' * (localcovar \ (ss-localmean));
      end

      clik  = exp(clik-max(clik)) .* (objnd(newoo,:) + alpha);
      newcc = multinomial_rnd(clik, 1);
    else
      prob = clik(ones(numobj,1),:);  % prob = repmat(clik,numobj,1);
      for oo = 1:numobj
        indA = dim*(oo-1) + 1;
        indB = dim*oo;

        for cc = 1:numclass
          localmean   = groupmean(indA:indB) + classmean(:,cc);
          localcovar  = groupcovar(indA:indB,indA:indB) + classcovar(:,:,cc);
          prob(oo,cc) = prob(oo,cc) - 0.5*log(det(localcovar)) - ...
            0.5*(ss-localmean)' * (localcovar \ (ss-localmean));
        end
      end

      prob     = exp(prob-max(max(prob))) .* (objnd + alpha);
      probnorm = (scenend+gamma) ./ sum(objnd+alpha,2);
      prob     = prob .* probnorm(:,ones(1,numclass));

      % allow exclusion of single object category
      if labeloo(ii) < 0
        prob(-labeloo(ii),:) = 0;
      end

      newoocc = multinomial_rnd(prob(:),1);
      newcc   = fix((newoocc-1)/numobj) + 1;
      newoo   = newoocc - (newcc-1)*numobj;
    end

    % add data item back to model
    datacc(ii) = newcc;
    dataoo(ii) = newoo;

    newref = groupmean(dim*(newoo-1)+1:dim*newoo);
    newcov = groupcovar(dim*(newoo-1)+1:dim*newoo,dim*(newoo-1)+1:dim*newoo);
    classsum(:,newcc)    = classsum(:,newcc) + (ss - newref);
    classout(:,:,newcc)  = classout(:,:,newcc) + newcov + (ss-newref)*(ss-newref)';
    groupsum(:,newoo,newcc) = groupsum(:,newoo,newcc) + ss;

    if usetypeB
      classqqB(:,newcc) = datastat_add(classqqB(:,newcc), ...
        classnd(newcc), ssB, dtype(2), dparamB);
    end

    classnd(newcc)       = classnd(newcc) + 1;
    objnd(newoo,newcc)   = objnd(newoo,newcc) + 1;
    scenend(newoo)       = scenend(newoo) + 1;
    groupnd(newoo,newcc) = groupnd(newoo,newcc) + 1;
  end

  % add current group back to reference position statistics
  refsum  = refsum + groupmean;
  refchol = cholupdate(refchol, groupmean);
  refchol = chol(refchol'*refchol + groupcovar);

  % cache all state variables associated only with current group (image)
  sceneobj.datacc{jj}            = datacc;
  sceneobj.dataoo{jj}            = dataoo;
  sceneobj.scenend(:,category)   = scenend;
  sceneobj.groupnd(:,:,jj)       = groupnd;
  sceneobj.groupsum(:,:,:,jj)    = groupsum;
  sceneobj.groupmean(:,jj)       = groupmean;
  sceneobj.groupcovar(:,:,jj)    = groupcovar;
  sceneobj.refsum(:,category)    = refsum;
  sceneobj.refchol(:,:,category) = refchol;
end

% store class state variables for next iteration
sceneobj.classnd    = classnd;
sceneobj.objnd      = objnd; 
if usetypeB
  sceneobj.classqq{2} = classqqB;
end

%DEBUGGING: Part statistic consistency following EM updates
if (0)
classsum_old = classsum; 
classout_old = classout;
classsum = zeros(dim,numclass);
classout = repmat(6*eye(dim),[1 1 numclass]);

for jj = 1:sceneobj.numgroup
  numdata   = sceneobj.numdata(jj);
  datass    = sceneobj.datass{jj}{1};
  datacc    = sceneobj.datacc{jj};
  dataoo    = sceneobj.dataoo{jj};

  groupmean  = sceneobj.groupmean(:,jj);
  groupcovar = sceneobj.groupcovar(:,:,jj);

  for ii = find(datacc > 0)
    ss = datass(:,ii);
    cc = datacc(ii);
    oo = dataoo(ii);

    ref    = groupmean(dim*(oo-1)+1:dim*oo);
    refcov = groupcovar(dim*(oo-1)+1:dim*oo,dim*(oo-1)+1:dim*oo);

    classsum(:,cc)   = classsum(:,cc) + (ss-ref);
    classout(:,:,cc) = classout(:,:,cc) + refcov + (ss-ref)*(ss-ref)';
  end
end
disp(max(max(abs(classsum-classsum_old))))
for cc = 1:numclass
  disp(max(max(abs(classout(:,:,cc)-classout_old(:,:,cc)))))
end
end

% convert Gaussian class statistics back to standard form
for cc = 1:numclass
  classout(:,:,cc) = chol(classout(:,:,cc));
  classout(:,:,cc) = cholupdate(classout(:,:,cc), ...
    classsum(:,cc)./sqrt(classkappa+classnd(cc)), '-');
end

sceneobj.classqq{1}(1:dim,:)       = classsum;
sceneobj.classqq{1}(dim+1:end-1,:) = reshape(classout, dim^2, numclass);
%TODO: Set logdet cache for compatibility with normal sampler???

