
function sceneobj = ...
  sceneobject(numgroup,numscene,numobj,numclass,alpha,gamma,dataprior,refprior)
%sceneobject     Models multi-object scenes via distributions over shared parts
%
%    SO = sceneobject(numgroup, numscene, numobj, numclass, ...
%                     alpha, gamma, dataprior, refprior)
%
%  Construct instance SO of sceneobject class, where
%    numgroup  = number of groups of data (images or documents)
%    numscene  = number of scene types (groups of objects)
%    numobj    = number of categories (objects or authors)
%    numclass  = number of latent classes (parts or topics)
%    alpha     = Dirichlet hyperparameter for category-specific 
%                distributions over classes
%    gamma     = Dirichlet hyperparameter for scene-specific 
%                distributions over categories
%    dataprior = struct of hyperparameters for class-specific feature
%                distributions, or cell array of structs if multiple datatypes
%                (see datastat_init.m for available types and parameters)
%    refprior  = struct of hyperparameters for reference position
%                distributions (requires first datatype to be Gaussian),
%                or empty to not use reference positions

  % Default constructor (needed so load has proper functionality)
  if nargin == 0
    numgroup = 1;
    numscene = 1;
    numobj   = 1;
    numclass = 1;
    alpha    = 1;
    gamma    = 1;
    refprior = [];

    dataprior.type       = 'discrete';
    dataprior.numfeature = 1;
    dataprior.dirweight  = 1;
  elseif nargin < 8
    refprior = [];
  end

  if ~iscell(dataprior)
    dataprior = {dataprior};
  end

  % Dirichlet prior parameters
  sceneobj.alpha = alpha;
  sceneobj.gamma = gamma;

  % Counts: groups, objects, classes/parts 
  sceneobj.numgroup = numgroup;
  sceneobj.numscene = numscene;
  sceneobj.numobj   = numobj;
  sceneobj.numclass = numclass;

  % Data items: counts, sufficient statistics, class/object assignments, categories
  sceneobj.numtype  = length(dataprior);
  sceneobj.numdata  = zeros(numgroup,1);
  sceneobj.datass   = cell(numgroup,1);
  sceneobj.datacc   = cell(numgroup,1);
  sceneobj.dataoo   = cell(numgroup,1);
  sceneobj.labeloo  = cell(numgroup,1);
  sceneobj.category = zeros(numgroup,1);

  for jj = 1:numgroup
    sceneobj.datass{jj} = cell(sceneobj.numtype,1);
  end

  % Object categories: data item counts
  sceneobj.objnd   = zeros(numobj,numclass); 
  sceneobj.scenend = zeros(numobj,numscene);

  % Classes: data item counts, sufficient statistics, hyperparameters
  sceneobj.classnd  = zeros(1,numclass);

  for dd = 1:sceneobj.numtype
    [qq0,dtype,dparam] = datastat_init(dataprior{dd});
    sceneobj.classqq{dd} = repmat(qq0, 1, numclass);
    sceneobj.dtype(dd)   = dtype;
    sceneobj.dparam{dd}  = dparam;
  end

  % Reference position statistics and hyperparameters
  sceneobj.sceneng = zeros(1,numscene);

  if isempty(refprior)
    sceneobj.useref = 0;

    sceneobj.rparam  = [];
    sceneobj.refsum  = [];
    sceneobj.refchol = [];

    sceneobj.groupnd    = [];
    sceneobj.groupsum   = [];
    sceneobj.groupmean  = [];
    sceneobj.groupcovar = [];
  else
    sceneobj.useref = 1;

    nu0    = refprior.ccovar;
    kappa0 = refprior.cmean;
    mu0    = refprior.emean;
    sigma0 = refprior.ecovar;
    dim    = round(length(mu0)/numobj);

    sceneobj.rparam  = [dim; nu0; kappa0];
    sceneobj.refsum  = repmat(kappa0*mu0,1,numscene);
    sceneobj.refchol = repmat(chol(nu0*sigma0), [1 1 numscene]);

    sceneobj.groupnd    = zeros(numobj,numclass,numgroup);
    sceneobj.groupsum   = zeros(dim,numobj,numclass,numgroup);
    sceneobj.groupmean  = zeros(dim*numobj,numgroup);
    sceneobj.groupcovar = zeros(dim*numobj,dim*numobj,numgroup);
  end

  % Instantiate class
  sceneobj = class(sceneobj,'sceneobject');

