
function tdp = gibbs_crf(tdp, groups, labels)
%gibbs_crf    Iteration of Chinese restaurant franchise Gibbs sampler

% determine number of datatypes used in data likelihoods
% hardcoded for 1 or 2 datatypes to avoid overhead of loop over datatypes
if tdp.numtype > 2
  error('Gibbs sampler supports at most two data types');
end
usetypeB = (tdp.numtype > 1);

% static parameters
gamma    = tdp.gamma;
alpha    = tdp.alpha;
numgroup = tdp.numgroup;

% class (part) level state variables
numclass = tdp.numclass;
classnd  = tdp.classnd;
classnt  = tdp.classnt;
classpp  = tdp.classpp;
classqq  = tdp.classqq{1};

% data parameters
dtype  = tdp.dtype;
dparam = tdp.dparam{1};
qq0    = tdp.qq0{1};

% parameters and state variables for second datatype
if usetypeB
  dparamB  = tdp.dparam{2};
  qq0B     = tdp.qq0{2};
  classqqB = tdp.classqq{2};
end

% transformation parameters
ttype    = tdp.ttype;
tparam   = tdp.tparam;
pp0      = tdp.pp0;

% add empty class to allow creation of new clusters
classnd(numclass+1)   = 0;
classnt(numclass+1)   = 0;
classpp(:,numclass+1) = pp0;
classqq(:,numclass+1) = qq0; 
if usetypeB
  classqqB(:,numclass+1) = qq0B;
end

% resample groups in random order
for j = randperm(length(groups))
  jj = groups(j);
  
  % table and data level state variables for current group
  numdata  = tdp.numdata(jj);
  numtable = tdp.numtable(jj);
  datass   = tdp.datass{jj}{1};
  datatt   = tdp.datatt{jj};
  datall   = tdp.datall{jj};
  tablecc  = tdp.tablecc{jj};
  tablerr  = tdp.tablerr{jj};
  tablend  = tdp.tablend{jj};

  % data items for second datatype
  if usetypeB
    datassB = tdp.datass{jj}{2};
  else
    datassB = zeros(0,numdata);
  end

  % sample table associated with each data item, in random order
  sampind = find(ismember(datall, labels{j}));
  for i = randperm(length(sampind))
    ii    = sampind(i);
    ss    = datass(:,ii);
    ssB   = datassB(:,ii);
    oldtt = datatt(ii);

    % temporarily remove data item from model, unless not yet assigned
    if oldtt > 0
      oldcc = tablecc(oldtt);

      if ttype > 0
        ssref = ss - tablerr(:,oldtt); % TODO: generic transforms
        classqq(:,oldcc) = datastat_del(classqq(:,oldcc), ...
          classnd(oldcc), ssref, dtype(1), dparam);
      else
        classqq(:,oldcc) = datastat_del(classqq(:,oldcc), ...
          classnd(oldcc), ss, dtype(1), dparam);
      end
      if usetypeB
        classqqB(:,oldcc) = datastat_del(classqqB(:,oldcc), ...
          classnd(oldcc), ssB, dtype(2), dparamB);
      end

      classnd(oldcc) = classnd(oldcc) - 1;
      tablend(oldtt) = tablend(oldtt) - 1;
    else
      oldcc = 0;
    end
    
    % compute conditional likelihood factors for each table
    if ttype > 0
      clik = zeros(1, numtable+numclass+1);

      % existing tables
      for tt = 1:numtable
        cc = tablecc(tt);
        ssref = ss - tablerr(:,tt); % TODO: generic transforms
        clik(tt) = datastat_condlike(classqq(:,cc), ...
          classnd(cc), ssref, dtype(1), dparam);
      end

      % approximately integrate over reference states to sample new tables
      clik(numtable+1:end) = trandata_marglike(classqq, classnd, ...
        classpp, classnt, ss, dtype(1), dparam, ttype, tparam);
    else  
      % reuse likelihood calculations for tables that share classes
      clikClass = datastat_condlike(classqq, classnd, ss, dtype(1), dparam);
      clik      = clikClass([tablecc 1:numclass+1]);
    end

    if usetypeB
      clikClass = datastat_condlike(classqqB, classnd, ssB, dtype(2), dparamB);
      clik      = clik + clikClass([tablecc 1:numclass+1]);
    end
    
    % compute Dirichlet process prior weights for each class
    if isinf(gamma)  
      weights = [zeros(1,numclass), 1];
    else
      weights = [classnt(1:numclass), gamma];
      weights = weights/sum(weights);
    end

    % sample a new table for this data point
    clik  = exp(clik-max(clik)) .* [tablend alpha*weights];
    newtt = multinomial_rnd(clik, 1);

    % add a new table if needed
    if newtt > numtable
      newcc          = newtt-numtable;
      newtt          = numtable + 1;
      numtable       = newtt;
      tablend(newtt) = 0;
      tablecc(newtt) = newcc;

      % add a new class if needed
      if newcc > numclass
        numclass           = newcc;
        classnd(newcc+1)   = 0;
        classnt(newcc+1)   = 0;
        classqq(:,newcc+1) = qq0;
        classpp(:,newcc+1) = pp0;
        if usetypeB
          classqqB(:,newcc+1) = qq0B;
        end
      end

      if ttype > 0
        tablerr(:,newtt) = trandata_samp(classqq(:,newcc), classnd(newcc), ...
          classpp(:,newcc), classnt(newcc), ss, dtype(1), dparam, ttype, tparam);
      else
        tablerr(:,newtt) = 0;
      end
      classpp(:,newcc) = transtat_add(classpp(:,newcc), ...
        classnt(newcc), tablerr(:,newtt), ttype, tparam);
      classnt(newcc)   = classnt(newcc) + 1;
    else
      newcc = tablecc(newtt); 
    end

    % add data item back to model
    if ttype > 0
      ssref = ss - tablerr(:,newtt); % TODO: generic transforms
      classqq(:,newcc) = datastat_add(classqq(:,newcc), ...
        classnd(newcc), ssref, dtype(1), dparam);
    else
      classqq(:,newcc) = datastat_add(classqq(:,newcc), ...
        classnd(newcc), ss, dtype(1), dparam);
    end
    if usetypeB
      classqqB(:,newcc) = datastat_add(classqqB(:,newcc), ...
        classnd(newcc), ssB, dtype(2), dparamB);
    end

    datatt(ii)     = newtt;
    classnd(newcc) = classnd(newcc) + 1;
    tablend(newtt) = tablend(newtt) + 1;
  end

  % remove any deserted tables
  for tt = numtable:-1:1
    if tablend(tt) == 0
      numtable = numtable - 1;
      cc = tablecc(tt);
      classpp(:,cc) = transtat_del(classpp(:,cc), ...
        classnt(cc), tablerr(:,tt), ttype, tparam);
      classnt(cc)   = classnt(cc) - 1;

      ii = find(datatt > tt);
      datatt(ii) = datatt(ii) - 1;
      tablecc(tt) = [];
      tablerr(:,tt) = [];
      tablend(tt) = [];
    end
  end

  % resample class and state associated with each table 
  if isfinite(gamma)
    for tt = randperm(numtable)
      ss    = datass(:,find(datatt==tt));
      ssB   = datassB(:,find(datatt==tt));
      oldcc = tablecc(tt);
      oldrr = tablerr(:,tt);

      % temporarily remove table from model
      if ttype > 0
        ssref = ss - oldrr(:,ones(1,size(ss,2))); % TODO: generic transforms
        classqq(:,oldcc) = datastat_del(classqq(:,oldcc), ...
          classnd(oldcc), ssref, dtype(1), dparam);
      else
        classqq(:,oldcc) = datastat_del(classqq(:,oldcc), ...
          classnd(oldcc), ss, dtype(1), dparam);
      end
      if usetypeB
        classqqB(:,oldcc) = datastat_del(classqqB(:,oldcc), ...
          classnd(oldcc), ssB, dtype(2), dparamB);
      end
      classpp(:,oldcc) = transtat_del(classpp(:,oldcc), ...
        classnt(oldcc), oldrr, ttype, tparam);
      classnd(oldcc)   = classnd(oldcc) - size(ss,2);
      classnt(oldcc)   = classnt(oldcc) - 1;

      % compute conditional data likelihood factors for each class
      if ttype > 0
        % approximately integrate over reference states to sample new tables
        clik = trandata_marglike(classqq, classnd, classpp, classnt, ss, ...
          dtype(1), dparam, ttype, tparam);
      else  
        clik = datastat_condlike(classqq, classnd, ss, dtype(1), dparam);
      end
      if usetypeB
        clik = clik + datastat_condlike(classqqB,classnd,ssB,dtype(2),dparamB);
      end

      % sample a new class for this table
      clik  = exp(clik-max(clik)) .* [classnt(1:numclass), gamma];
      newcc = multinomial_rnd(clik, 1);

      % sample a new state corresponding to this class
      if ttype > 0
        newrr = trandata_samp(classqq(:,newcc), classnd(newcc), ...
          classpp(:,newcc), classnt(newcc), ss, dtype(1), dparam, ttype, tparam);
      else
        newrr = 0;
      end

      % add a new class if needed
      if newcc > numclass
        numclass           = newcc;
        classnd(newcc+1)   = 0;
        classnt(newcc+1)   = 0;
        classqq(:,newcc+1) = qq0;
        classpp(:,newcc+1) = pp0;
        if usetypeB
          classqqB(:,newcc+1) = qq0B;
        end
      end

      % add table back to model
      tablecc(tt)      = newcc;
      tablerr(:,tt)    = newrr;
      if ttype > 0
        ssref = ss - newrr(:,ones(1,size(ss,2))); % TODO: generic transforms
        classqq(:,newcc) = datastat_add(classqq(:,newcc), ...
          classnd(newcc), ssref, dtype(1), dparam);
      else
        classqq(:,newcc) = datastat_add(classqq(:,newcc), ...
          classnd(newcc), ss, dtype(1), dparam);
      end
      if usetypeB
        classqqB(:,newcc) = datastat_add(classqqB(:,newcc), ...
          classnd(newcc), ssB, dtype(2), dparamB);
      end

      classpp(:,newcc) = transtat_add(classpp(:,newcc), ...
        classnt(newcc), newrr, ttype, tparam);
      classnd(newcc)   = classnd(newcc) + size(ss,2);
      classnt(newcc)   = classnt(newcc) + 1;
    end
  end

  tdp.numtable(jj) = numtable;
  tdp.datatt{jj}   = datatt;
  tdp.tablecc{jj}  = tablecc;
  tdp.tablerr{jj}  = tablerr;
  tdp.tablend{jj}  = tablend;
end

% remove empty dummy class 
classnd(numclass+1)   = [];
classnt(numclass+1)   = [];
classpp(:,numclass+1) = [];
classqq(:,numclass+1) = [];
if usetypeB
  classqqB(:,numclass+1) = [];
end

% remove any unused classes
for cc = numclass:-1:1
  if classnt(cc) == 0
    numclass      = numclass - 1;
    classnd(cc)   = [];
    classnt(cc)   = [];
    classpp(:,cc) = [];
    classqq(:,cc) = [];
    if usetypeB
      classqqB(:,cc) = [];
    end

    for jj = 1:numgroup
      tt = find(tdp.tablecc{jj} > cc);
      tdp.tablecc{jj}(tt) = tdp.tablecc{jj}(tt) - 1;
    end
  end
end

tdp.numclass   = numclass;
tdp.classnd    = classnd;
tdp.classnt    = classnt;
tdp.classpp    = classpp;
tdp.classqq{1} = classqq;
if usetypeB
  tdp.classqq{2} = classqqB;
end

