
function tdp = sampledata(tdp,groups,numsamp,maxclass,maxtable,label,thetaqq,thetapp)
%sampledata     Generate random datasets using specified, or random, parameters
%
%    TDP = sampledata(TDP, groups, numsamp, maxclass, maxtable, label)
%
%  Generate synthetic dataset by random sampling, where
%    groups   = groups for which synthetic data created (all, by default)
%    numsamp  = number of new data items sampled for each group
%    maxclass = maximum number of shared classes instantiated by
%               sampled data (zero for no limit, the default)
%    maxtable = maximum number of transformed classes (tables) created
%               within each group (zero for no limit, the default)
%    label    = optional label assigned to data items (by default, label=0)

% Default parameters
if nargin < 2
  groups = [];
end
if nargin < 3
  numsamp = 100;
end
if nargin < 4
  maxclass = 0;
end
if nargin < 5
  maxtable = 0;
end
if nargin < 6
  label = 0;
end
if nargin < 7
  thetaqq = {};
elseif ~iscell(thetaqq)
  thetaqq = {thetaqq};
end
if nargin < 8
  thetapp = [];
end

if isempty(groups)
  groups = [1:tdp.numgroup];
end

% static parameters
gamma    = tdp.gamma;
alpha    = tdp.alpha;
numgroup = tdp.numgroup;

% class (part) level state variables
numclass = tdp.numclass;
classnd  = tdp.classnd;
classnt  = tdp.classnt;
classpp  = tdp.classpp;
classqq  = tdp.classqq;

% data parameters
numtype = tdp.numtype;
dtype   = tdp.dtype;
dparam  = tdp.dparam;
qq0     = tdp.qq0;

% transformation parameters
ttype   = tdp.ttype;
tparam  = tdp.tparam;
pp0     = tdp.pp0;

% determine parameters associated with existing classes
if isempty(thetaqq)
  thetaqq = cell(numtype,1);
  for dd = 1:numtype
    thetaqq{dd} = datastat_estparam(classqq{dd}, classnd, dtype(dd), dparam{dd});
  end
  thetapp = transtat_estparam(classpp, classnt, ttype, tparam);
else
  if length(thetaqq) ~= numtype
    error('Invalid parameters');
  end
  if size(thetaqq{1},2) < numclass
    error('Must specify parameters for all existing classes');
  end
  for dd = 2:numtype
    if size(thetaqq{dd},2) ~= size(thetaqq{1},2)
      error('Internal inconsistency in thetaqq');
    end
  end
end

% Sample dataset for each specified group
for j = 1:length(groups)
  jj = groups(j);

  % table and data level state variables for current group
  numtable = tdp.numtable(jj);
  tablecc  = tdp.tablecc{jj};
  tablerr  = tdp.tablerr{jj};
  tablend  = tdp.tablend{jj};

  % Independently sample each data item
  newdatass = cell(numtype,1);
  newdatatt = zeros(1,numsamp);
  for ii = 1:numsamp
    % sample table associated with data item
    if (maxtable > 0) && (numtable >= maxtable)
      newtt = multinomial_rnd(tablend, 1);
    else
      newtt = multinomial_rnd([tablend alpha], 1);
    end

    % add a new table if needed
    if newtt > numtable
      % sample class associated with new table
      if (maxclass > 0) && (numclass >= maxclass)
        newcc = multinomial_rnd(classnt, 1);
      else
        if isinf(gamma)
          newcc = numclass + 1;
        else 
          newcc = multinomial_rnd([classnt gamma], 1);
        end
      end

      % add a new class if needed
      if newcc > numclass
        numclass       = newcc;
        classnd(newcc) = 0;
        classnt(newcc) = 0;

        for dd = 1:numtype
          classqq{dd}(:,newcc) = qq0{dd};
          if newcc > size(thetaqq{dd},2)
            thetaqq{dd}(:,newcc) = datastat_sampparam(qq0{dd}, ...
              0, 1, dtype(dd), dparam{dd});
          end
        end

        classpp(:,newcc) = pp0;
        if newcc > size(thetapp,2)
          thetapp(:,newcc) = transtat_sampparam(pp0, 0, 1, ttype, tparam);
        end
      end

      % sample transformation for new table
      tablerr(:,newtt) = tranparam_samptran(thetapp(:,newcc), 1, ttype, tparam);
      classpp(:,newcc) = transtat_add(classpp(:,newcc), ...
        classnt(newcc), tablerr(:,newtt), ttype, tparam);

      % update statistics to account for new table
      numtable       = newtt;
      tablend(newtt) = 0;
      tablecc(newtt) = newcc;
      classnt(newcc) = classnt(newcc) + 1;
    else
      newcc = tablecc(newtt); 
    end

    % sample data item using table/class parameters, and update statistics
    for dd = 1:numtype
      newdatass{dd}(:,ii)  = dataparam_sampdata(thetaqq{dd}(:,newcc), ...
        1, dtype(dd), dparam{dd});
      classqq{dd}(:,newcc) = datastat_add(classqq{dd}(:,newcc), ...
        classnd(newcc), newdatass{dd}(:,ii), dtype(dd), dparam{dd});

      % TODO: support generic transformations
      if ttype > 0 && dd == 1
        newdatass{dd}(:,ii) = newdatass{dd}(:,ii) + tablerr(:,newtt);
      end
    end
    newdatatt(ii)  = newtt;
    classnd(newcc) = classnd(newcc) + 1;
    tablend(newtt) = tablend(newtt) + 1;
  end

  % store data, updated tables, and generating table assignments
  tdp.numdata(jj)  = tdp.numdata(jj) + numsamp;
  tdp.numtable(jj) = numtable;
  tdp.tablecc{jj}  = tablecc;
  tdp.tablerr{jj}  = tablerr;
  tdp.tablend{jj}  = tablend;
  tdp.datatt{jj}   = [tdp.datatt{jj}, newdatatt];
  tdp.datall{jj}   = [tdp.datall{jj}, label*ones(1,numsamp)];
  for dd = 1:numtype
    tdp.datass{jj}{dd} = [tdp.datass{jj}{dd}, newdatass{dd}];
  end
end

% store updated classes
tdp.numclass = numclass;
tdp.classnd  = classnd;
tdp.classnt  = classnt;
tdp.classpp  = classpp;
tdp.classqq  = classqq;

