
function tdp = tdpmix(numgroup, alpha, gamma, dataprior, tranprior)
%tdpmix     Models grouped data via distributions over transformed mixture components
%
%    TDP = tdpmix(numgroup, alpha, gamma, dataprior, tranprior)
%
%  Construct instance TDP of tdpmix class, where
%    numgroup  = number of groups of data (images or documents)
%    alpha     = Dirichlet process hyperparameter for group-specific mixtures,
%                or two-element vector specifying gamma prior distribution
%    gamma     = Dirichlet process hyperparameter for global mixture,
%                or two-element vector specifying gamma prior distribution
%                (gamma=inf gives independent DP mixture for each group)
%    dataprior = struct of hyperparameters for base measure on data parameters,
%                or cell array of structs if multiple datatypes
%                (see datastat_init.m for available types and parameters)
%    tranprior = struct of hyperparameters for base measure on transformations,
%                assumed to apply to first data type, or empty for standard HDP
%                (see transtat_init.m for available types and parameters)

  % Default constructor (needed so load has proper functionality)
  if nargin == 0
    numgroup  = 1;
    alpha     = 1;
    gamma     = 1;
    tranprior = struct([]);

    dataprior.type       = 'discrete';
    dataprior.numfeature = 1;
    dataprior.dirweight  = 1;
  elseif nargin < 5
    tranprior = struct([]);
  end

  if ~iscell(dataprior)
    dataprior = {dataprior};
  end

  % Dirichlet process hyperparameters
  if length(gamma) == 1
    tdp.gamma      = gamma;
    tdp.gammaprior = [gamma 1];
  elseif length(gamma) == 2
    tdp.gamma      = gamma(1)/gamma(2);
    tdp.gammaprior = gamma;
  else
    error('Invalid global concentration parameter gamma');
  end

  if length(alpha) == 1
    tdp.alpha      = alpha;
    tdp.alphaprior = [alpha 1];
  elseif length(alpha) == 2
    tdp.alpha      = alpha(1)/alpha(2);
    tdp.alphaprior = alpha;
  else
    error('Invalid group concentration parameter alpha');
  end

  % Counts 
  tdp.numgroup = numgroup;          % number of groups
  tdp.numtable = zeros(numgroup,1); % number of tables in each group
  tdp.numclass = 0;                 % number of global classes/parts/topics

  % Data items 
  tdp.numtype  = length(dataprior); % number of types of data or features
  tdp.numdata  = zeros(numgroup,1); % number of data items in each group
  tdp.datass   = cell(numgroup,1);  % data vectors
  tdp.datatt   = cell(numgroup,1);  % assignments of data to tables
  tdp.datall   = cell(numgroup,1);  % label assigned to each data item

  for jj = 1:numgroup
    tdp.datass{jj} = cell(tdp.numtype,1);
  end

  % Tables 
  tdp.tablecc = cell(numgroup,1);   % assignments of tables to classes
  tdp.tablend = cell(numgroup,1);   % number of data items at each table
  tdp.tablerr = cell(numgroup,1);   % transformation for each table

  % Classes
  tdp.classnd = zeros(1,0);         % number of data items for each class
  tdp.classnt = [];                 % number of tables for each class

  for dd = 1:tdp.numtype            % cached statistics for each datatype
    [qq0,dtype,dparam] = datastat_init(dataprior{dd});
    tdp.qq0{dd}     = qq0;
    tdp.classqq{dd} = zeros(length(qq0),0);
    tdp.dtype(dd)   = dtype;
    tdp.dparam{dd}  = dparam;
  end

  % Classes: transformation statistics
  if isempty(tranprior)
    tdp.pp0     = 0;
    tdp.classpp = zeros(1,0);
    tdp.ttype   = 0;
    tdp.tparam  = [];
  else
    [pp0,ttype,tparam] = transtat_init(tranprior);
    tdp.pp0     = pp0;
    tdp.classpp = zeros(length(pp0),0);
    tdp.ttype   = ttype;
    tdp.tparam  = tparam;
  end

  % Instantiate class
  tdp = class(tdp,'tdpmix');

