        
function visualize(tdp, groups, plotstyle, Hfig)
%visualize     Visualize posterior distributions implied by current assignments
%
%    visualize(TDP, groups, plotstyle, Hfig)
%
%  Visualize classes implied by current assignments, where 
%    groups    = groups in which to visualize transformed classes, or tables
%                (maximum of five)
%    plotstyle = cell array of structs of datatype-specific parameters
%    Hfig      = vector of figure handles for each datatype (if empty, 
%                a new figure is created for each datatype)

if nargin < 2
  groups = [];
end
if nargin < 3
  plotstyle = struct([]);
end
if nargin < 4
  Hfig = [];
end

dtype   = tdp.dtype;
dparam  = tdp.dparam;
numtype = tdp.numtype;
ttype   = tdp.ttype;

if ~iscell(plotstyle)
  tempstyle = plotstyle;
  plotstyle = cell(numtype,1);

  for dd = 1:numtype
    plotstyle{dd} = tempstyle;
  end
end

% extract current class statistics
numclass = tdp.numclass;
classqq  = tdp.classqq;
classnd  = tdp.classnd;
classnt  = tdp.classnt;

% create figures, if needed
if isempty(Hfig)
  for dd = 1:numtype
    Hfig(dd) = figure;
  end
end

% preprocess variables needed to plot tables for each group
numplot = length(groups);
if numplot > 5
  warning('Only displaying tables for first five groups');
  groups = groups(1:5);
  numplot = 5;
end
numrows = ceil(numplot/2);

weightsT = cell(numplot,1);
for j = 1:numplot
  jj = groups(j);
  tablecc  = tdp.tablecc{jj};
  tablend  = tdp.tablend{jj};
  numtable = tdp.numtable(jj);

  if (ttype > 0)
    weightsT{j} = tablend/sum(tablend);
  else
    weightsT{j} = zeros(1,numclass);
    for tt = 1:numtable
      weightsT{j}(tablecc(tt)) = weightsT{j}(tablecc(tt)) + tablend(tt);
    end
    weightsT{j} = weightsT{j}/sum(weightsT{j});
  end
end

for dd = 1:numtype
  % Posterior parameters for each class
  theta = datastat_estparam(classqq{dd}, classnd, dtype(dd), dparam{dd});

  if numplot > 0
    Hax = subplot(numrows,2,1,'Parent',Hfig(dd));
  else
    Hax = subplot(1,1,1,'Parent',Hfig(dd));
  end

  % Discrete multinomial
  if dtype(dd)==1,   
    imagesc(theta,'Parent',Hax);
    colormap(Hax,gray(256));

    for j = 1:numplot
      if (ttype > 0)
        tablecc = tdp.tablecc{groups(j)};
        thetaT  = theta(:,tablecc);
      else
        usedclass = find(weightsT{j} > 0);
        thetaT    = theta(:,usedclass);
      end

      Hax = subplot(numrows,2,j+1,'Parent',Hfig(dd));
      imagesc(thetaT,'Parent',Hax);
      colormap(Hax,gray(256));
    end
  
  % Normal-Inverse-Wishart
  elseif (dtype(dd)==2) || (dtype(dd)==3 && dparam{dd}(1)==2),   
    dim   = dparam{dd}(1);
    mu    = theta(1:dim,:);
    if dtype(dd) == 2
      sigma = reshape(theta(dim+1:end,:),dim,dim,numclass);
    else
      sigma = zeros(dim,dim,numclass);
      for cc = 1:numclass
        sigma(:,:,cc) = diag(theta(dim+1:end,cc));
      end
    end

    % realign mean values to account for empirical mean of current tables
    muTables = zeros(dim,numclass);
    for jj = 1:tdp.numgroup
      tablecc  = tdp.tablecc{jj};
      tablerr  = tdp.tablerr{jj};
      numtable = tdp.numtable(jj);

      for tt = 1:numtable
        muTables(:,tablecc(tt)) = muTables(:,tablecc(tt)) + tablerr(:,tt);
      end
    end
    muTables = muTables ./ classnt(ones(dim,1),:);
  
    cla(Hax);
    if isfield(plotstyle{dd}, 'numbers') && (plotstyle{dd}.numbers > 0)
      plotGaussMix(mu+muTables, sigma, classnt/sum(classnt), 'numbers', Hax);
    else
      plotGaussMix(mu+muTables, sigma, classnt/sum(classnt), [], Hax);
    end
    if isfield(plotstyle{dd}, 'range')
      axis(Hax, plotstyle{dd}.range);
    end

    for j = 1:numplot
      if (ttype > 0)
        tablecc = tdp.tablecc{groups(j)};

        usedclass = [1:length(tablecc)]; 
        if dd==1
          muT = mu(:,tablecc) + tdp.tablerr{groups(j)};
        else
          muT = mu(:,tablecc);
        end
        sigmaT = sigma(:,:,tablecc);
      else
        usedclass = find(weightsT{j} > 0);
        muT       = mu(:,usedclass);
        sigmaT    = sigma(:,:,usedclass);
      end

      Hax = subplot(numrows,2,j+1,'Parent',Hfig(dd));
      cla(Hax);
      if isfield(plotstyle{dd}, 'numbers') && (plotstyle{dd}.numbers > 0)
        plotGaussMix(muT, sigmaT, weightsT{j}(usedclass), 'numbers', Hax);
      else
        plotGaussMix(muT, sigmaT, weightsT{j}(usedclass), [], Hax);
      end
      if isfield(plotstyle{dd}, 'range')
        axis(Hax, plotstyle{dd}.range);
      end
    end

  % Diagonal Normal-Inverse-Wishart
  elseif dtype(dd)==3,   
    dim   = dparam{dd}(1);
    mu    = theta(1:dim,:);
    sigma = theta(dim+1:end,:);  

    if isfield(plotstyle{dd}, 'meanscale')
      meanscale = plotstyle{dd}.meanscale;
    else
      meanscale = [min(mu(:)) max(mu(:))];
    end
    sigma = sigma ./ max(sigma(:));
    sigma = sigma.*(meanscale(2)-meanscale(1)) + meanscale(1);
    imagesc([mu; sigma],'Parent',Hax);
    colormap(Hax,gray(256));

    for j = 1:numplot
      if (ttype > 0)
        tablecc = tdp.tablecc{groups(j)};
        muT     = mu(:,tablecc);
        sigmaT  = sigma(:,tablecc);
      else
        usedclass = find(weightsT{j} > 0);
        muT       = mu(:,usedclass);
        sigmaT    = sigma(:,usedclass);
      end

      Hax = subplot(numrows,2,j+1,'Parent',Hfig(dd));
      imagesc([muT; sigmaT],'Parent',Hax);
      colormap(Hax,gray(256));
    end

  end
end

