
function tdp = ...
  doGibbs(tdp,numiter,auxiter,visparam,sampgroup,coniter,sampcat,noGlobal,anneal)
%doGibbs     Approximates posterior distributions via Gibbs sampling
%
%  TDP=doGibbs(TDP,numiter,auxiter,visparam,sampgroup,coniter,sampcat,noGlobal,anneal)
%
%  Learning and inference via Gibbs sampling, where
%    numiter   = number of iterations (one iteration resamples all 
%                assignments of data to objects, in all chosen groups, once)
%    auxiter   = number of auxiliary iterations used to determine likelihoods
%                for reassigning tables to new objects (by default, two)
%    visparam  = cell array of arguments to visualize, or empty for
%                no plot of learning process
%    sampgroup = groups on which to run Gibbs sampling (by default, all)
%    coniter   = number of concentration parameter resampling iterations
%                between each Gibbs sampling iteration (by default, zero)
%    sampcat   = user-specified categories which should be resampled, or
%                empty to resample all features (default)
%    noGlobal  = set to true to not allow sampler to create new global
%                clusters (false, by default)
%    anneal    = optional annealing of likelihoods used to reassign tables
%                to global categories; either a numiter-dim. annealing schedule
%                or a single value, where 0 denotes no annealing and 1 is
%                a sensible "starting" annealing

  % default parameters
  if nargin < 2
    numiter = 1;
  end
  if nargin < 3
    auxiter = 2;
  end
  if nargin < 4
    visparam = [];
  end
  if nargin < 5 
    sampgroup = [1:tdp.numgroup];
  end
  if nargin < 6
    coniter = 0;
  end
  if nargin < 7
    sampcat = [];
  end
  if nargin < 8
    noGlobal = 0;
  end
  if nargin < 9
    anneal = 0;
  end

  if length(anneal) == 1
    anneal = anneal*ones(numiter,1);
  end

  for iter = 1:numiter
    % table and global cluster assignments
    if (tdp.ztype > 0)
      tdp = gibbs_crf3D(tdp, sampgroup, auxiter, noGlobal, anneal(iter), sampcat);
    else
      tdp = gibbs_crf2D(tdp, sampgroup, auxiter, noGlobal, anneal(iter), sampcat);
    end

    % concentration parameters
    numobj = tdp.numobj;
    if coniter > 0

      if isfinite(tdp.gamma)
        tdp.gamma = gibbs_conparam(tdp.gamma, sum(tdp.numtable), numobj, ...
          tdp.gammaprior(1), tdp.gammaprior(2), coniter);
      end

      % must only include groups where data has been assigned
      validgroup = find(tdp.numtable > 0);  
      groupdata  = zeros(length(validgroup),1); 
      for j = 1:length(validgroup)
        groupdata(j) = sum(tdp.datatt{validgroup(j)} ~= 0);
      end
      tdp.alpha  = gibbs_conparam(tdp.alpha, groupdata, tdp.numtable(validgroup), ...
        tdp.alphaprior(1), tdp.alphaprior(2), coniter);

      % resample based on currently instantiated object categories
      if tdp.conpartprior(1) > 0
        objdata = zeros(numobj,1);
        for cc = 1:numobj
          objdata(cc) = sum(tdp.partnd{cc});
        end
        tdp.conpart = gibbs_conparam(tdp.conpart, objdata, tdp.numpart', ...
          tdp.conpartprior(1), tdp.conpartprior(2), coniter);
      end
    end

    if ~isempty(visparam)
      visualize(tdp, visparam{:});
      drawnow;
    end

    fprintf(1,'%d(%d %.3f %.3f %.3f).',iter,numobj,tdp.alpha,tdp.gamma,tdp.conpart);
  end
  fprintf(1,'\n');

