
function objProb = estassign(tdp, groups, method)
%estassign     Estimate object category probabilities for each feature
%
%     objProb = estassign(TDP, groups, method)
%
%  Estimate posterior probability that each feature comes from each user-specified
%  object category, averaging across visual categories, where
%     groups   = indexes of groups to estimate categories (by default, all)
%     method   = scheme used to estimate object category probabilities
%                0 -> integrate over table assignments (default)
%                1 -> naive counting of table assignments 
%     objProb  = cell array such that objProb{j}(:,i) gives estimated
%                object category probabilities for feature i in groups(j),
%                where objProb{j}(end,i) denotes the probability for 
%                visual categories to which no user-labeled features are assigned
%  Ignores any user-specified labels for individual features, to allow
%  consistency tests for training images.

if nargin < 2
  groups = [];
end
if nargin < 3
  method = 0;
end

numgroup = tdp.numgroup;
if isempty(groups)
  groups = [1:numgroup];
end

% determine set of user-specified labels in current global clusters
objll  = max(tdp.objll,0);  % ignore any negative labels
objcat = unique(objll);

% determine probability of non-labeled categories
if objcat(1) == 0
  objcat = [objcat(2:end), 0];
else
  objcat = [objcat, 0];
end
numcat = length(objcat);

objProb = cell(length(groups),1);
for j=1:length(groups)
  objProb{j} = zeros(numcat, tdp.numdata(groups(j)));
end

% part state variables
numpart = tdp.numpart;
partnd  = tdp.partnd;
partqq  = tdp.partqq;

% data parameters
dtype  = tdp.dtype;
dparam = tdp.dparam;

% depth parameters
ztype = tdp.ztype;

for j=1:length(groups)
  jj = groups(j);

  % table and data level state variables for current group
  numdata  = tdp.numdata(jj);
  numtable = tdp.numtable(jj);
  datass   = tdp.datass{jj}{1};
  dataoo   = tdp.datass{jj}{2};
  datakk   = tdp.datakk{jj};
  datatt   = abs(tdp.datatt{jj});
  tablecc  = tdp.tablecc{jj};
  tablerr  = tdp.tablerr{jj};
  tablend  = tdp.tablend{jj};

  if method == 1
    for ii = 1:numdata
      if datatt(ii) > 0
        category = objll(tablecc(datatt(ii))); 
        if category == 0
          category = numcat
        end

        objProb{j}(category,ii) = 1;
      else
        objProb{j}(:,ii) = 1/numcat;
      end
    end
  else
    if ztype > 0    
      datass(1:2,:) = datass(1:2,:) .* datass([3;3],:); % project to 3D
    end

    % conditional likelihoods and part probabilities for existing tables
    ttind  = [0 cumsum(numpart(tablecc))]; 
    cliktt = zeros(numdata, ttind(end));
    probtt = zeros(1, ttind(end));

    for tt = 1:numtable
      cc = tablecc(tt);

      % probability of each part, normalized by overall table probability
      probtt(ttind(tt)+1:ttind(tt+1)) = partnd{cc} * (tablend(tt)/sum(partnd{cc}));
      
      % appearance and position likelihoods
      ssref = datass - tablerr(:,tt*ones(1,numdata)); % TODO: generic transforms
      cliktt(:,ttind(tt)+1:ttind(tt+1)) = ...
        datastat_indeplike(partqq{cc}{1}, partnd{cc}, ssref, dtype(1), dparam{1});
      cliktt(:,ttind(tt)+1:ttind(tt+1)) = cliktt(:,ttind(tt)+1:ttind(tt+1)) + ...
        datastat_indeplike(partqq{cc}{2}, partnd{cc}, dataoo, dtype(2), dparam{2});
    end
    probtt = probtt'/sum(probtt);

    % compute posterior probability of each table/part assignment for each feature
    maxlike  = max(cliktt');
    maxlike  = maxlike(ones(ttind(end),1),:);
    postprob = exp(cliktt'-maxlike) .* probtt(:,ones(1,numdata));
    probnorm = sum(postprob);
    postprob = postprob ./ probnorm(ones(ttind(end),1),:);

    for tt = 1:numtable
      category = objll(tablecc(tt));
      if category == 0
        category = numcat;
      end

      objProb{j}(category,:) = objProb{j}(category,:) + ...
        sum(postprob(ttind(tt)+1:ttind(tt+1),:),1);
    end
  end
end

