
function tdp = gibbs_crf3D(tdp, groups, numaux, noGlobal, annealLike, sampcat)
%gibbs_crf3D    Iteration of 3D Chinese restaurant franchise Gibbs sampler

% static parameters
gamma    = tdp.gamma;
alpha    = tdp.alpha;
conpart  = tdp.conpart;
numgroup = tdp.numgroup;

% optionally restrict creation of new global clusters, and new parts
if noGlobal
  gamma   = 0;
  conpart = 0;
end

% feature position/appearance parameters
stype  = tdp.dtype(1);
sparam = tdp.dparam{1};
sqq0   = tdp.qq0{1};
otype  = tdp.dtype(2);
oparam = tdp.dparam{2};
oqq0   = tdp.qq0{2};

% transformation parameters
ttype  = tdp.ttype;
tparam = tdp.tparam;
pp0    = tdp.pp0;

% depth parameters
ztype  = tdp.ztype;
zparam = tdp.zparam;
[zgrid,zwidth] = depth_getgrid(ztype, zparam);
numzz  = length(zgrid);

% object state variables
numobj  = tdp.numobj;
objnt   = tdp.objnt;
objpp   = tdp.objpp;
objll   = tdp.objll;
numpart = tdp.numpart;

% part state variables: add empty part to each category
partnd = cell(1,numobj+1);
posqq  = cell(1,numobj+1);
obsqq  = cell(1,numobj+1);
for cc = 1:numobj
  partnd{cc} = [tdp.partnd{cc}, 0];
  posqq{cc}  = [tdp.partqq{cc}{1}, sqq0];
  obsqq{cc}  = [tdp.partqq{cc}{2}, oqq0];
end

% add empty visual object category
objnt(numobj+1)   = 0;
objpp(:,numobj+1) = pp0;
objll(numobj+1)   = 0;
numpart(numobj+1) = 0;
partnd{numobj+1}  = 0;
posqq{numobj+1}   = sqq0;
obsqq{numobj+1}   = oqq0;

% resample groups in random order
nnn=0;
for j = randperm(length(groups))
  jj = groups(j);
  
  % update visualizations more often
  nnn=nnn+1;
  if mod(nnn,10)==0 
    drawnow; 
  end
  fprintf(1,'.'); 

  % table and data level state variables for current group
  numtable = tdp.numtable(jj);
  datass   = tdp.datass{jj}{1};
  dataoo   = tdp.datass{jj}{2};
  datazz   = tdp.datass{jj}{3};
  datakk   = tdp.datakk{jj};
  datatt   = tdp.datatt{jj};
  datall   = tdp.datall{jj};
  tablecc  = tdp.tablecc{jj};
  tablerr  = tdp.tablerr{jj};
  tablend  = tdp.tablend{jj};

  % sample table associated with each data item, in random order
  % do not sample data items with manually fixed table assignments,
  % and allow restriction to features with specified category labels
  if length(sampcat) > 0
    sampmask = zeros(1,tdp.numdata(jj));
    for ll = sampcat
      sampmask(datall==ll) = 1; 
    end
    sampind = find((datatt >= 0) & sampmask);
  else
    sampind = find(datatt >= 0);
  end

  for i = randperm(length(sampind))
    ii    = sampind(i);
    ss    = datass(:,ii);
    oo    = dataoo(:,ii);
    zz    = datazz(:,ii);
    ll    = datall(ii);
    oldkk = datakk(ii);
    oldtt = datatt(ii);

    % temporarily remove data item from model, unless not yet assigned
    if oldtt > 0
      oldcc = tablecc(oldtt);

      ssref = [ss(1:2); 1]*ss(3) - tablerr(:,oldtt); % TODO: generic transforms
      posqq{oldcc}(:,oldkk) = datastat_del(posqq{oldcc}(:,oldkk), ...
        partnd{oldcc}(oldkk), ssref, stype, sparam);
      obsqq{oldcc}(:,oldkk) = datastat_del(obsqq{oldcc}(:,oldkk), ...
        partnd{oldcc}(oldkk), oo, otype, oparam);
      partnd{oldcc}(oldkk)  = partnd{oldcc}(oldkk) - 1;
      tablend(oldtt) = tablend(oldtt) - 1;
    else
      oldcc = 0;
    end
    
    % account for data points with labeled object categories
    % assumes at most one category is associated with negative labels
    if ll > 0
      validtt = find((objll(tablecc) == ll) | (objll(tablecc) == 0) | ...
                    ((objll(tablecc) ~=-ll) & (objll(tablecc) < 0)));
      validcc = find((objll == ll) | (objll == 0) | ...
                    ((objll ~=-ll) & (objll < 0)));
    elseif ll < 0 
      validtt = find(objll(tablecc) ~= -ll);
      validcc = find(objll ~= -ll);
    else
      validtt = [1:numtable];
      validcc = [1:numobj+1];
    end

    % data likelihoods and object/part probabilities for potential new tables
    ccind  = [0 cumsum(numpart(validcc)+1)]; 
    clikcc = zeros(numzz, ccind(end));
    clikoo = zeros(1, ccind(end));
    probcc = zeros(1, ccind(end));

    for c = 1:length(validcc)
      cc = validcc(c);

      % probability of each of this object's parts, allowing for conpart=0
      probcc(ccind(c)+1:ccind(c+1)) = [partnd{cc}(1:numpart(cc)), conpart];
      normcc = sum(probcc(ccind(c)+1:ccind(c+1)));
      if normcc > 0
        probcc(ccind(c)+1:ccind(c+1)) = probcc(ccind(c)+1:ccind(c+1)) / normcc;
      else
        probcc(ccind(c)+1:ccind(c+1)) = [zeros(1,numpart(cc)), 1];
      end

      % compute appearance likelihoods, and save for reuse by existing tables
      clikoo(ccind(c)+1:ccind(c+1)) = ...
        datastat_condlike(obsqq{cc}, partnd{cc}, oo, otype, oparam);
      clikcc(:,ccind(c)+1:ccind(c+1)) = clikoo(ones(numzz,1),ccind(c)+1:ccind(c+1));

      % integrate over reference transforms associated with new table
      clikcc(:,ccind(c)+1:ccind(c+1)) = clikcc(:,ccind(c)+1:ccind(c+1)) + ... 
        depth_marglike(posqq{cc}, partnd{cc}, objpp(:,cc*ones(1,numpart(cc)+1)), ...
          objnt(cc*ones(1,numpart(cc)+1)), ss, stype, sparam, ...
          ttype, tparam, ztype, zparam);
    end
    clikcc = clikcc + zz(:,ones(1,ccind(end)));

    % conditional likelihoods and part probabilities for existing tables
    ttind  = [0 cumsum(numpart(tablecc(validtt))+1)]; 
    cliktt = zeros(numzz, ttind(end));
    probtt = zeros(1, ttind(end));

    for t = 1:length(validtt)
      tt = validtt(t);
      cc = tablecc(tt);
      c  = find(validcc == cc);

      % probability of each part, normalized by overall table probability
      probtt(ttind(t)+1:ttind(t+1)) = probcc(ccind(c)+1:ccind(c+1)) * tablend(tt);
      
      % appearance and position likelihoods
      cliktt(:,ttind(t)+1:ttind(t+1)) = ...
        clikoo(ones(numzz,1),ccind(c)+1:ccind(c+1)) + ...
        depth_condlike(posqq{cc}, partnd{cc}, ...
          tablerr(:,tt*ones(1,numpart(cc)+1)), ss, stype, sparam, ztype, zparam);
    end
    cliktt = cliktt + zz(:,ones(1,ttind(end)));

    % normalize probability of new tables versus existing tables
    for c = 1:length(validcc)
      cc = validcc(c);
      if cc <= numobj
        probcc(ccind(c)+1:ccind(c+1)) = probcc(ccind(c)+1:ccind(c+1)) * objnt(cc);
      else
        probcc(ccind(c)+1:ccind(c+1)) = probcc(ccind(c)+1:ccind(c+1)) * gamma;
      end
    end
    probcc = probcc * (alpha/sum(probcc));
    
    % sample a new table, part, and depth for this data point
    clik      = [cliktt clikcc];
    weights   = [probtt probcc];
    %TODO: the exp() and max() are slow below; alternatives?
    clik      = exp(clik-max(clik(:))) .* weights(ones(numzz,1),:);
    newzzttkk = multinomial_rnd(clik(:), 1);
    newttkk   = fix((newzzttkk-1)/numzz) + 1;
    newzz     = newzzttkk - (newttkk-1)*numzz;
    ss(3)     = zgrid(newzz) + zwidth(newzz)*(2*rand(1)-1); 

    % add a new table if needed
    if newttkk > length(probtt)
      newcckk = newttkk - length(probtt);
      newcc   = max(find(ccind < newcckk));
      newkk   = newcckk - ccind(newcc);
      newcc   = validcc(newcc);

      newtt          = numtable + 1;
      numtable       = newtt;
      tablend(newtt) = 0;
      tablecc(newtt) = newcc;

      % add a new object category if needed
      if newcc > numobj
        numobj = newcc;

        objnt(numobj+1)   = 0;
        objpp(:,numobj+1) = pp0;
        objll(numobj+1)   = 0;  
        numpart(numobj+1) = 0;
        partnd{numobj+1}  = 0;
        posqq{numobj+1}   = sqq0;
        obsqq{numobj+1}   = oqq0;
      end

      % add a new part if needed
      if newkk > numpart(newcc)
        numpart(newcc) = newkk;
        partnd{newcc}  = [partnd{newcc}, 0];
        posqq{newcc}   = [posqq{newcc}, sqq0];
        obsqq{newcc}   = [obsqq{newcc}, oqq0];
      end

      % sample transformation for new table 
      ssref = [ss(1:2); 1]*ss(3); % project to 3D
      tablerr(:,newtt) = trandata_samp(posqq{newcc}(:,newkk), ...
        partnd{newcc}(newkk), objpp(:,newcc), objnt(newcc), ...
        ssref, stype, sparam, ttype, tparam);
      objpp(:,newcc)   = transtat_add(objpp(:,newcc), objnt(newcc), ...
        tablerr(:,newtt), ttype, tparam);
      objnt(newcc)     = objnt(newcc) + 1;
    else
      newtt = max(find(ttind < newttkk));
      newkk = newttkk - ttind(newtt);
      newtt = validtt(newtt);
      newcc = tablecc(newtt);

      % add a new part if needed
      if newkk > numpart(newcc)
        numpart(newcc) = newkk;
        partnd{newcc}  = [partnd{newcc}, 0];
        posqq{newcc}   = [posqq{newcc}, sqq0];
        obsqq{newcc}   = [obsqq{newcc}, oqq0];
      end
    end

    % add data item back to model
    ssref = [ss(1:2); 1]*ss(3) - tablerr(:,newtt); % TODO: generic transforms
    posqq{newcc}(:,newkk) = datastat_add(posqq{newcc}(:,newkk), ...
      partnd{newcc}(newkk), ssref, stype, sparam);
    obsqq{newcc}(:,newkk) = datastat_add(obsqq{newcc}(:,newkk), ...
      partnd{newcc}(newkk), oo, otype, oparam);
    partnd{newcc}(newkk)  = partnd{newcc}(newkk) + 1;

    datakk(ii)     = newkk;
    datatt(ii)     = newtt;
    tablend(newtt) = tablend(newtt) + 1;

    % store newly resampled depth
    datass(3,ii) = ss(3);

    % update object label for this category, if necessary
    if ll > 0
      objll(newcc) = ll;
    elseif (ll < 0) && (objll(newcc) == 0)
      objll(newcc) = ll;
    end
  end

  % remove any deserted tables
  for tt = numtable:-1:1
    if tablend(tt) == 0
      numtable = numtable - 1;
      cc = tablecc(tt);
      objpp(:,cc) = transtat_del(objpp(:,cc), objnt(cc), ...
        tablerr(:,tt), ttype, tparam);
      objnt(cc)   = objnt(cc) - 1;

      ii = find(datatt > tt);
      datatt(ii)    = datatt(ii) - 1;
      tablecc(tt)   = [];
      tablerr(:,tt) = [];
      tablend(tt)   = [];
    end
  end

  % resample object category, transformation, parts, and depths for each table
  % exclude tables not matching specified category labels
  if length(sampcat) > 0
    sampmask = zeros(1,numtable);
    for ll = sampcat
      sampmask(objll(tablecc)==ll) = 1; 
    end
    samptable = find(sampmask);
  else
    samptable = [1:numtable];
  end

  for t = randperm(length(samptable))
    tt    = samptable(t);
    ttind = find(abs(datatt)==tt);
    ss    = datass(:,ttind);
    oo    = dataoo(:,ttind);
    zz    = datazz(:,ttind);
    ll    = datall(ttind);
    oldkk = datakk(ttind);
    oldcc = tablecc(tt);
    oldrr = tablerr(:,tt);
    numdata = length(ttind);

    % temporarily remove table from model
    ssref = [ss(1:2,:).*ss([3;3],:); ss(3,:)]; % project to 3D
    ssref = ssref - oldrr(:,ones(1,numdata));  % TODO: generic transforms
    for ii = 1:numdata
      posqq{oldcc}(:,oldkk(ii)) = datastat_del(posqq{oldcc}(:,oldkk(ii)), ...
        partnd{oldcc}(oldkk(ii)), ssref(:,ii), stype, sparam);
      obsqq{oldcc}(:,oldkk(ii)) = datastat_del(obsqq{oldcc}(:,oldkk(ii)), ...
        partnd{oldcc}(oldkk(ii)), oo(:,ii), otype, oparam);
      partnd{oldcc}(oldkk(ii))  = partnd{oldcc}(oldkk(ii)) - 1;
    end
    objpp(:,oldcc) = transtat_del(objpp(:,oldcc), objnt(oldcc), ...
      oldrr, ttype, tparam);
    objnt(oldcc)   = objnt(oldcc) - 1;

    % account for data points with labeled object categories
    posll = ll(find(ll > 0));
    negll = ll(find(ll < 0));
    if length(posll) > 0
      % all positive labels agree, assuming no bugs above
      validcc = find((objll == posll(1)) | (objll == 0) | ...
                    ((objll ~=-posll(1)) & (objll < 0)));
    elseif length(negll) > 0
      % assumes at most one category is associated with negative labels
      validcc = find(objll ~= -negll(1));
    else
      validcc = [1:numobj+1];
    end

    % approximate likelihood of each potential category via Gibbs sampler
    clikcc = zeros(1,length(validcc));
    refcc  = zeros(size(ss,1),length(validcc)); 
    ssref  = [ss(1:2,:).*ss([3;3],:); ss(3,:)]; % project to 3D
    for c = 1:length(validcc)
      cc = validcc(c);
      [clikcc(c),refcc(:,c)] = gibbs_llikTable(numaux, ssref, oo, conpart, ...
        partnd{cc}, posqq{cc}, obsqq{cc}, objpp(:,cc), objnt(cc), ...
        stype, sparam, otype, oparam, ttype, tparam);
    end

    % annealing to make table reassignments easier
    clikcc = clikcc / (size(ss,2)^annealLike);

    % sample a new object category and transformation for this table
    clik  = exp(clikcc-max(clikcc)) .* [objnt(validcc(1:end-1)), gamma];
    newcc = multinomial_rnd(clik, 1);
    newrr = refcc(:,newcc);
    newcc = validcc(newcc);

    % add a new object category if needed
    if newcc > numobj
      numobj = newcc;

      objnt(numobj+1)   = 0;
      objpp(:,numobj+1) = pp0;
      objll(numobj+1)   = 0;  
      numpart(numobj+1) = 0;
      partnd{numobj+1}  = 0;
      posqq{numobj+1}   = sqq0;
      obsqq{numobj+1}   = oqq0;
    end

    % sample new assignments of features to parts, and feature depths
    for ii = randperm(numdata)
      % likelihood of each part and depth
      clikoo = datastat_condlike(obsqq{newcc}, partnd{newcc}, ...
        oo(:,ii), otype, oparam);
      clik   = clikoo(ones(numzz,1),:) + depth_condlike(posqq{newcc}, ...
        partnd{newcc}, newrr(:,ones(1,numpart(newcc)+1)), ss(:,ii), ...
        stype, sparam, ztype, zparam);
      clik   = clik + zz(:,ii*ones(1,numpart(newcc)+1));

      % prior probability of each part, allowing for conpart=0
      probkk = [partnd{newcc}(1:numpart(newcc)), conpart];
      normkk = sum(probkk);
      if normkk > 0
        probkk = probkk/normkk;
      else
        probkk = [zeros(1,numpart(newcc)), 1];
      end

      % sample part assignment and depth according to these probabilities
      clik     = exp(clik-max(clik(:))) .* probkk(ones(numzz,1),:);
      newzzkk  = multinomial_rnd(clik(:), 1);
      newkk    = fix((newzzkk-1)/numzz) + 1;
      newzz    = newzzkk - (newkk-1)*numzz;
      ss(3,ii) = zgrid(newzz) + zwidth(newzz)*(2*rand(1)-1); 

      % add a new part if needed
      if newkk > numpart(newcc)
        numpart(newcc) = newkk;
        partnd{newcc}  = [partnd{newcc}, 0];
        posqq{newcc}   = [posqq{newcc}, sqq0];
        obsqq{newcc}   = [obsqq{newcc}, oqq0];
      end

      % add feature to statistics of chosen part
      ssref = [ss(1:2,ii); 1]*ss(3,ii) - newrr; % TODO: generic transforms
      posqq{newcc}(:,newkk) = datastat_add(posqq{newcc}(:,newkk), ...
        partnd{newcc}(newkk), ssref, stype, sparam);
      obsqq{newcc}(:,newkk) = datastat_add(obsqq{newcc}(:,newkk), ...
        partnd{newcc}(newkk), oo(:,ii), otype, oparam);
      partnd{newcc}(newkk)  = partnd{newcc}(newkk) + 1;

      datakk(ttind(ii)) = newkk;
    end

    % add table back to model
    tablecc(tt)    = newcc;
    tablerr(:,tt)  = newrr;
    objpp(:,newcc) = transtat_add(objpp(:,newcc), objnt(newcc), ...
      newrr, ttype, tparam);
    objnt(newcc)   = objnt(newcc) + 1;

    % store newly resampled depths
    datass(3,ttind) = ss(3,:);

    % update object label for this category, if necessary
    if length(posll) > 0
      objll(newcc) = posll(1);
    elseif (length(negll) > 0) && (objll(newcc) == 0)
      objll(newcc) = negll(1);
    end
  end

  % store updated depth estimates
  tdp.datass{jj}{1} = datass;

  tdp.numtable(jj) = numtable;
  tdp.datakk{jj}   = datakk;
  tdp.datatt{jj}   = datatt;
  tdp.tablecc{jj}  = tablecc;
  tdp.tablerr{jj}  = tablerr;
  tdp.tablend{jj}  = tablend;
end

% remove empty dummy object category
objnt(numobj+1)   = [];
objpp(:,numobj+1) = [];
objll(numobj+1)   = [];
numpart(numobj+1) = [];
partnd(numobj+1)  = [];
posqq(numobj+1)   = [];
obsqq(numobj+1)   = [];

for cc = numobj:-1:1
  % remove empty dummy part from each object category
  partnd{cc}(numpart(cc)+1)  = [];
  posqq{cc}(:,numpart(cc)+1) = [];
  obsqq{cc}(:,numpart(cc)+1) = [];

  % remove any unused parts from each object category
  for kk = numpart(cc):-1:1
    if partnd{cc}(kk) == 0
      numpart(cc) = numpart(cc) - 1;
      partnd{cc}(kk)  = [];
      posqq{cc}(:,kk) = [];
      obsqq{cc}(:,kk) = [];

      for jj = 1:numgroup
        tablecc = tdp.tablecc{jj};
        partind = find(tdp.datakk{jj} > kk);
        ii = find(tablecc(abs(tdp.datatt{jj}(partind))) == cc);
        tdp.datakk{jj}(partind(ii)) = tdp.datakk{jj}(partind(ii)) - 1;
      end
    end
  end

  % remove any unused object categories
  if objnt(cc) == 0
    if numpart(cc) > 0
      error('Internal inconsistency between objnt and numpart');
    end

    numobj      = numobj - 1;
    objnt(cc)   = [];
    objpp(:,cc) = [];
    objll(cc)   = [];
    numpart(cc) = [];
    partnd(cc)  = [];
    posqq(cc)   = [];
    obsqq(cc)   = [];

    for jj = 1:numgroup
      tt = find(tdp.tablecc{jj} > cc);
      tdp.tablecc{jj}(tt) = tdp.tablecc{jj}(tt) - 1;
    end
  end
end

% rebuild global object labels, in case features which induced a label removed
newll = zeros(size(objll));
for jj = 1:numgroup
  tablecc  = tdp.tablecc{jj};
  datatt   = abs(tdp.datatt{jj});
  datall   = tdp.datall{jj};
  labelind = find((datatt > 0) & (datall ~= 0));

  for ii = labelind
    cc = tablecc(datatt(ii));       

    if datall(ii) > 0
      newll(cc) = datall(ii);
    elseif newll(cc) == 0
      newll(cc) = datall(ii);
    end
  end
end

% object state variables
tdp.numobj  = numobj;
tdp.objnt   = objnt;
tdp.objpp   = objpp;
tdp.objll   = newll;
tdp.numpart = numpart;

% part state variables 
tdp.partnd = partnd;
tdp.partqq = cell(1,numobj);
for cc = 1:numobj
  tdp.partqq{cc}{1} = posqq{cc};
  tdp.partqq{cc}{2} = obsqq{cc};
end

