
function tdp = tdpscenepart(numgroup, alpha, gamma, nu, dataprior, tranprior)
%tdpscenepart    Transformed DP model for 2D/3D scenes with objects & parts
%
%    TDP = tdpscenepart(numimage, alpha, gamma, nu, dataprior, tranprior)
%
%  Construct instance TDP of tdpscenepart class, where
%    numimage  = number of images (training and test)
%    alpha     = Dirichlet process hyperparameter for object instances (tables),
%                or two-element vector specifying gamma prior distribution
%    gamma     = Dirichlet process hyperparameter for global object categories,
%                or two-element vector specifying gamma prior distribution
%    nu        = Dirichlet process hyperparameter for object parts,
%                or two-element vector specifying gamma prior distribution
%    dataprior = cell array where dataprior{1} is struct of hyperparameters
%                for feature positions, dataprior{2} is struct of
%                hyperparameters for feature appearance, and dataprior{3}
%                is struct of parameters for depth observations, or
%                empty for a 2D model based on image coordinates
%                (see datastat_init.m for available types and parameters)
%    tranprior = struct of hyperparameters for base measure on transformations,
%                assumed to apply to first (feature position) data type

%Sample parameters for 3D model:
%dataprior{1}.type   = 'gaussian';
%dataprior{1}.emean  = zeros(3,1);
%dataprior{1}.cmean  = 0.1;
%dataprior{1}.ecovar = 2*eye(3);
%dataprior{1}.ccovar = 8;
%dataprior{2}.type   = 'discrete';
%dataprior{2}.numfeature = 500;
%dataprior{2}.dirweight  = 0.1;
%dataprior{3}.grid   = linspace(1,10,100)';
%dataprior{3}.width  = 0.5*(dataprior{3}.grid(2)-dataprior{3}.grid(1))*ones(100,1);
%tranprior.type   = 'zeromeangaussian';
%tranprior.ccovar = 8;
%tranprior.ecovar = 10*eye(3);

  % Default constructor (needed so load has proper functionality)
  if nargin == 0
    numgroup  = 1;
    alpha     = 1;
    gamma     = 1;
    nu        = 1;
    tranprior = struct([]);

    dataprior.type       = 'discrete';
    dataprior.numfeature = 1;
    dataprior.dirweight  = 1;
    dataprior = {dataprior,dataprior};

    %depthparam.grid  = linspace(0,1,10)';
    %depthparam.width = (depthparam.grid(2)-depthparam.grid(1))*ones(10,1);
    %dataprior = {dataprior,dataprior,depthparam};
  elseif nargin < 6
    tranprior = struct([]);
  end

  if ~iscell(dataprior) || (length(dataprior) < 2)
    error('Must provide hyperparameters for both position and appearance');
  end

  % Dirichlet process hyperparameters
  if length(gamma) == 1
    tdp.gamma      = gamma;
    tdp.gammaprior = [gamma 1];
  elseif length(gamma) == 2
    tdp.gamma      = gamma(1)/gamma(2);
    tdp.gammaprior = gamma;
  else
    error('Invalid global concentration parameter gamma');
  end

  if length(alpha) == 1
    tdp.alpha      = alpha;
    tdp.alphaprior = [alpha 1];
  elseif length(alpha) == 2
    tdp.alpha      = alpha(1)/alpha(2);
    tdp.alphaprior = alpha;
  else
    error('Invalid group concentration parameter alpha');
  end

  if length(nu) == 1
    tdp.conpart      = nu;
    tdp.conpartprior = [nu 1];
  elseif length(nu) == 2
    tdp.conpart      = nu(1)/nu(2);
    tdp.conpartprior = nu;
  else
    error('Invalid part concentration parameter nu');
  end

  % Images
  tdp.numgroup = numgroup;          % number of groups
  tdp.numtable = zeros(numgroup,1); % number of tables in each group

  % Data items 
  tdp.numtype  = length(dataprior); % number of types of data or features
  tdp.numdata  = zeros(numgroup,1); % number of data items in each group
  tdp.datass   = cell(numgroup,1);  % data vectors
  tdp.datakk   = cell(numgroup,1);  % assignments of data to parts
  tdp.datatt   = cell(numgroup,1);  % assignments of data to tables
  tdp.datall   = cell(numgroup,1);  % label assigned to each data item

  for jj = 1:numgroup
    tdp.datass{jj} = cell(tdp.numtype,1);
  end

  % Tables 
  tdp.tablecc = cell(numgroup,1);   % assignments of tables to objects
  tdp.tablend = cell(numgroup,1);   % number of data items at each table
  tdp.tablerr = cell(numgroup,1);   % transformation for each table

  % Object categories
  tdp.numobj = 0;                   % number of global visual object categories
  tdp.objnt = zeros(1,0);           % number of tables for each category
  tdp.objll = zeros(1,0);           % assignment constraints for each category
  tdp.numpart = zeros(1,0);         % number of parts for each object category

  % Parts (object-specific)
  tdp.partnd = cell(1,0);           % number of features assigned to each part
  tdp.partqq = cell(1,0);           % statistics of features for all parts

  % Cached statistics for feature position and appearance
  for ss = 1:2
    [qq0,dtype,dparam] = datastat_init(dataprior{ss});
    tdp.qq0{ss}    = qq0;
    tdp.dtype(ss)  = dtype;
    tdp.dparam{ss} = dparam;
  end

  % Parameters for evaluation of feature depth likelihoods
  if (length(dataprior) == 2) || isempty(dataprior{3})
    tdp.ztype  = 0;
    tdp.zparam = [];
  else
    [ztype,zparam] = depth_init(dataprior{3});
    tdp.ztype  = ztype;
    tdp.zparam = zparam;
  end

  % Feature position transformation statistics
  if isempty(tranprior)
    tdp.pp0    = 0;
    tdp.objpp  = zeros(1,0);
    tdp.ttype  = 0;
    tdp.tparam = [];
  else
    [pp0,ttype,tparam] = transtat_init(tranprior);
    tdp.pp0    = pp0;
    tdp.objpp  = zeros(length(pp0),0);
    tdp.ttype  = ttype;
    tdp.tparam = tparam;
  end

  % Instantiate class
  tdp = class(tdp,'tdpscenepart');

