
% Script for TDP/HDP/DP (transformed/shared/unshared) models for object categories

% 16-category experiment
savename = 'shareTDP';  % path and filename prefix for results
load sharing_ijcv16;    % precomputed features for all images

rand('state', 1); % seed to work always with same training/test data
randinitState = 1; % seed to allow different Gibbs sampler initializations

%%%%%%%%%%%%%
%% Parameters
%%%%%%%%%%%%%

useApp = 1; % appearance features: 0=none, 1=discrete, 2=Gaussian (diagonal)
usePos = 1; % position features:   0=none, 1=Gaussian, 2=transformed Gaussian

% Clustering hyperparameters
gamma  = [5   0.1];  % global concentration parameter (inf for unshared)
alpha  = [0.1 0.1];  % group-level concentration parameter 
numobj = imagedata.numclasses;

% Learning parameters
numburnin  = 200;
numsample  = 41;
numspace   = 20;
numconiter = 50;
numtrainO  = [4 6 8 10 12 15 20 30];
numtrainB  = 100;        

% Testing parameters
testspace = 1;
testindex = [1:2:41];

% Gaussian hyperparameters and visualization (less important)
dataprior{1}.type   = 'gaussian';
dataprior{1}.emean  = zeros(2,1)+64;
dataprior{1}.cmean  = 0.1;
dataprior{1}.ecovar = 200*eye(2); % square of diameter of the part
dataprior{1}.ccovar = 6;
plotstyle{1}.range  = [0 128 0 128];

% Discrete hyperparameters and visualization (less important)
dataprior{2}.type = 'discrete';
dataprior{2}.numfeature = imagedata.numwords; 
dataprior{2}.dirweight  = 0.1;    % beta in fig 1
plotstyle{2} = struct([]);

% Diagonal Gaussian appearance hyperparameters and visualization
appdim = 128;  % dimension of diagonal Gaussian appearance features
dataprior{3}.type   = 'diagonalgaussian';
dataprior{3}.emean  = zeros(appdim,1);
dataprior{3}.cmean  = 0.1;
dataprior{3}.ecovar = ones(appdim, 1);
dataprior{3}.ccovar = appdim+5;  % must be larger than appdim+1
plotstyle{3}.meanscale  = [-10 10];
plotstyle{3}.stdevscale = [0.5 2];

% Transformation hyperparameters
tranprior.type   = 'zeromeangaussian';
tranprior.ccovar = 6;
tranprior.ecovar = 1000*eye(2);

%%%%%%%%%%%%%%%%%
%% Initialization
%%%%%%%%%%%%%%%%%

numimage = length(imagedata.files);

% Select appropriate data hyperparameters
if useApp==1
  dataprior = dataprior(1:2);
  plotstyle = plotstyle(1:2);
elseif useApp==2
  dataprior = dataprior([1 3]);
  plotstyle = plotstyle([1 3]);
else
  dataprior = dataprior(1);
  plotstyle = plotstyle(1);
end
if usePos==0
  dataprior = dataprior(2);
  plotstyle = plotstyle(2);
  tranprior = struct([]);
elseif usePos==1
  tranprior = struct([]);
end

% Initialize object and background hierarchical models 
DPinfB  = tdpmix(1, alpha, inf, dataprior, []);
DPinitO = tdpmix(numobj, alpha, gamma, dataprior, tranprior);

% Use increasing subsets of same images when training
ind  = find(imagedata.classes == 0);
perm = randperm(length(ind));
trainglobalB = ind(perm(1:numtrainB));

for oo = 1:numobj
  ind  = find(imagedata.classes == oo);  
  perm = randperm(length(ind)); 
  trainglobalO(oo,:) = ind(perm(1:max(numtrainO)));
end

%%%%%%%%%%%%%%%%%%%
%% Train Background
%%%%%%%%%%%%%%%%%%%

% set Gibbs sampler initialization
rand('state',randinitState);

% training samples from background category
traingroup = sort(trainglobalB);

% insert training images into background model
for j = 1:length(traingroup)
  jj = traingroup(j);

  % encode data for this image
  if usePos
    x = imagedata.position{jj}(1,:) + 1; % add 1 for C-->Matlab coordinates
    y = imagedata.position{jj}(2,:) + 1; % add 1 for C-->Matlab coordinates
    newdata{1} = [x; y];
    if useApp==1
      newdata{2} = imagedata.appearance{jj};
    elseif useApp==2
      newdata{2} = log(double(Descriptor{jj})+1)-3;
    end
  else
    if useApp==1
      newdata{1} = imagedata.appearance{jj};
    elseif useApp==2
      newdata{1} = log(double(Descriptor{jj})+1)-3;
    end
  end

  % insert data into appropriate model(s)
  DPinfB = insertdata(DPinfB, newdata, 1, j);
end

% Gibbs sampling: initial 
fprintf(1, 'Burnin sampling for %d iterations...\n', numburnin);
DPinfB = doGibbs(DPinfB, numburnin, [], 1, [], numconiter);

% Store posterior assignments for later performance analysis
backhist = cell(1,numsample);

for ii = 1:numsample
  fprintf(1, 'Collecting sample %d of %d...\n', ii, numsample);
  backhist{ii} = DPinfB;

  if ii < numsample
    DPinfB = doGibbs(DPinfB, numspace, [], 1, [], numconiter);
  end
end

% Save all current results
save(sprintf('%s_back',savename), 'backhist', 'num*', ...
  'use*', 'alpha', 'gamma', 'dataprior', 'tranprior', 'plotstyle', 'savename');

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Consider different training set sizes
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
trainhist = cell(length(numtrainO),1);

for kk = 1:length(numtrainO)
%%%%%%%%%%%%%%%%
%% Train Objects
%%%%%%%%%%%%%%%%
  fprintf(1, 'Training with %d examples...\n', numtrainO(kk)); 

  % training samples from each object category
  traingroup = [];
  for oo = 1:numobj
    traingroup = [traingroup trainglobalO(oo,1:numtrainO(kk))];
  end

  % initialize models and image indices
  DPinfO = DPinitO;
  indexO = ones(numobj,1);

  % insert training images into models
  for jj = traingroup

    % encode data for this image
    if usePos
      x = imagedata.position{jj}(1,:) + 1; % add 1 for C-->Matlab coordinates
      y = imagedata.position{jj}(2,:) + 1; % add 1 for C-->Matlab coordinates
      newdata{1} = [x; y];
      if useApp==1
        newdata{2} = imagedata.appearance{jj};
      elseif useApp==2
        newdata{2} = log(double(Descriptor{jj})+1)-3;
      end
    else
      if useApp==1
        newdata{1} = imagedata.appearance{jj};
      elseif useApp==2
        newdata{1} = log(double(Descriptor{jj})+1)-3;
      end
    end

    % insert data into appropriate model(s)
    category = imagedata.classes(jj);
    DPinfO   = insertdata(DPinfO, newdata, category, indexO(category));
    indexO(category) = indexO(category) + 1; 
  end

  % test samples (both objects and background)
  traingroup = sort([traingroup trainglobalB]);
  testgroup  = setdiff([1:numimage], traingroup);
  testgroup  = testgroup(1:testspace:end);

  % Gibbs sampling: initial 
  fprintf(1, 'Burnin sampling for %d iterations...\n', numburnin);
  DPinfO = doGibbs(DPinfO, numburnin, [], [1:numobj], [], numconiter);

  % Store posterior assignments for later performance analysis
  trainhist{kk}.traingroup = traingroup;
  trainhist{kk}.testgroup  = testgroup;
  trainhist{kk}.DPinfO     = cell(1,numsample);

  for ii = 1:numsample
    fprintf(1, 'Collecting sample %d of %d...\n', ii, numsample);
    trainhist{kk}.DPinfO{ii} = DPinfO;

    if ii < numsample
      DPinfO = doGibbs(DPinfO, numspace, [], [1:numobj], [], numconiter);
    end
  end

  % Save all current results
  save(sprintf('%s_train%.2d',savename,numtrainO(kk)), 'backhist', 'trainhist', ...
    'num*', 'use*', 'alpha', 'gamma', 'dataprior', 'tranprior', 'plotstyle', ...
    'savename');

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Test: Detection and Recognition
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  numtest = length(testgroup);
  probO   = zeros(numobj+1,numtest,numsample);

  fprintf(1, 'Trained with %d, testing with %d: ', numtrainO(kk), numtest); 

  % evaluate posterior likelihood of each test image
  for j = 1:numtest
    jj = testgroup(j);

    % encode data for this image
    if usePos
      x = imagedata.position{jj}(1,:) + 1; % add 1 for C-->Matlab coordinates
      y = imagedata.position{jj}(2,:) + 1; % add 1 for C-->Matlab coordinates
      testdata{1} = [x; y];
      if useApp==1
        testdata{2} = imagedata.appearance{jj};
      elseif useApp==2
        testdata{2} = log(double(Descriptor{jj})+1)-3;
      end
    else
      if useApp==1
        testdata{1} = imagedata.appearance{jj};
      elseif useApp==2
        testdata{1} = log(double(Descriptor{jj})+1)-3;
      end
    end

    % evaluate each saved posterior model sample
    for ll = testindex
      DPinfB  = backhist{ll};
      DPinfO  = trainhist{kk}.DPinfO{ll};

      probO(1:numobj,j,ll) = sum(predictlike(DPinfO, [1:numobj], testdata),2);
      probO(numobj+1,j,ll) = sum(predictlike(DPinfB, 1, testdata));
    end

    fprintf(1, '%d.', j);
  end
  fprintf(1,'\n');

  % Store computed posterior probabilities
  trainhist{kk}.probO = probO;

  % Save all current results
  save(sprintf('%s_test%.2d',savename,numtrainO(kk)), 'backhist', 'trainhist', ...
    'num*', 'use*', 'alpha', 'gamma', 'dataprior', 'tranprior', 'plotstyle', ...
    'savename', 'testindex');
end

