
% Script for object-part parametric (LDA/author-topic) models for object categories
% Look at performance versus number of parts, for a single training set

% 16-category experiment
savename = 'partsLDA';  % path and filename prefix for results
load sharing_ijcv16;    % precomputed features for all images

rand('state', 1);  % seed to work always with same training/test data
randinitState = 1; % seed to allow different Gibbs sampler initializations

%%%%%%%%%%%%%
%% Parameters
%%%%%%%%%%%%%

useApp = 1; % appearance features: 0=none, 1=discrete, 2=Gaussian (diagonal)
usePos = 1; % 1 to use Gaussian feature position model, 0 neglects positions

% Clustering hyperparameters
numobj     = imagedata.numclasses;
numclassS  = [2:2:30]*numobj;  % number of shared object parts
numclassNS = [2:2:30];         % number of unshared object parts, per category
numclassB  = 30;        % number of background category parts 
alpha0     = 5;         % base clustering rate, divided equally among parts

% Learning parameters
numgibbs  = 200;          % number of iterations of Gibbs sampling
testiter  = [100:20:200]; % sampling iterations at which to evaluate test data
numtrainO = 15;      
numtrainB = 100;        

% Gaussian hyperparameters and visualization (less important)
dataprior{1}.type   = 'gaussian';
dataprior{1}.emean  = zeros(2,1)+64;
dataprior{1}.cmean  = 0.1;
dataprior{1}.ecovar = 200*eye(2); % square of diameter of the part
dataprior{1}.ccovar = 6;
plotstyle{1}.range  = [0 128 0 128];

% Discrete hyperparameters and visualization (less important)
dataprior{2}.type = 'discrete';
dataprior{2}.numfeature = imagedata.numwords; 
dataprior{2}.dirweight  = 0.1;    % Beta in fig 1
plotstyle{2} = struct([]);

% Diagonal Gaussian appearance hyperparameters and visualization
appdim = 128;  % dimension of diagonal Gaussian appearance features
dataprior{3}.type   = 'diagonalgaussian';
dataprior{3}.emean  = zeros(appdim,1);
dataprior{3}.cmean  = 0.1;
dataprior{3}.ecovar = ones(appdim, 1);
dataprior{3}.ccovar = appdim+5;  % must be larger than appdim+1
plotstyle{3}.meanscale  = [-10 10];
plotstyle{3}.stdevscale = [0.5 2];

%%%%%%%%%%%%%%%%%
%% Initialization
%%%%%%%%%%%%%%%%%

numimage = length(imagedata.files);

% Select appropriate data hyperparameters
if useApp==1
  dataprior = dataprior(1:2);
  plotstyle = plotstyle(1:2);
elseif useApp==2
  dataprior = dataprior([1 3]);
  plotstyle = plotstyle([1 3]);
else
  dataprior = dataprior(1);
  plotstyle = plotstyle(1);
end
if ~usePos
  dataprior = dataprior(2);
  plotstyle = plotstyle(2);
end

% Use increasing subsets of same images when training
ind  = find(imagedata.classes == 0);
perm = randperm(length(ind));
trainglobalB = ind(perm(1:numtrainB));

for oo = 1:numobj
  ind  = find(imagedata.classes == oo);  
  perm = randperm(length(ind)); 
  trainglobalO(oo,:) = ind(perm(1:max(numtrainO)));
end

%%%%%%%%%%%%%%%%%%%
%% Train Background
%%%%%%%%%%%%%%%%%%%

% set Gibbs sampler initialization
rand('state',randinitState);

% initialize background object-part hierarchy
OPinfB = objectpart(numimage, 1, numclassB, alpha0/numclassB, dataprior);

% insert training & test images into background model
for jj = 1:numimage
  category = imagedata.classes(jj);

  % encode data for this image
  if usePos
    x = imagedata.position{jj}(1,:) + 1; % add 1 for C-->Matlab coordinates
    y = imagedata.position{jj}(2,:) + 1; % add 1 for C-->Matlab coordinates
    newdata{1} = [x; y];
    if useApp==1
      newdata{2} = imagedata.appearance{jj};
    elseif useApp==2
      newdata{2} = log(double(Descriptor{jj})+1)-3;
    end
  else
    if useApp==1
      newdata{1} = imagedata.appearance{jj};
    elseif useApp==2
      newdata{1} = log(double(Descriptor{jj})+1)-3;
    end
  end

  % insert data into appropriate model(s)
  OPinfB = insertdata(OPinfB, newdata, jj, category==0);
end

% Gibbs sampling: background model
fprintf(1, 'Background sampling for %d iterations...\n', numgibbs);
[OPinfB,catprobB] = doGibbs(OPinfB,numgibbs,[],trainglobalB,[1:numimage],testiter);

% Save all current results
save(sprintf('%s_back',savename), 'catprobB', 'num*', 'use*', 'alpha0', ...
  'dataprior', 'plotstyle', 'savename', 'testiter', 'OPinfB');

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Consider different training set sizes
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

trainhist = cell(length(numclassS),1);

for zz = 1:length(numclassS)
  % training samples from background and each object category
  traingroup = trainglobalB;
  for oo = 1:numobj
    traingroup = [traingroup trainglobalO(oo,1:numtrainO)];
  end

  % test samples (both objects and background)
  traingroup = sort(traingroup);
  testgroup  = setdiff([1:numimage], traingroup);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Train Shared Object Model
%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  fprintf(1, 'Training shared model with %d parts...\n', numclassS(zz)); 

  % initialize shared object-part hierarchies
  OPinfS = objectpart(numimage,numobj,numclassS(zz),alpha0/numclassS(zz),dataprior);

  % insert data into shared and unshared models
  for jj = 1:numimage
    category = imagedata.classes(jj);
  
    % Insert appropriate data for each image
    if usePos
      x = imagedata.position{jj}(1,:) + 1; % add 1 for C-->Matlab coordinates
      y = imagedata.position{jj}(2,:) + 1; % add 1 for C-->Matlab coordinates
      newdata{1} = [x; y];
      if useApp==1
        newdata{2} = imagedata.appearance{jj};
      elseif useApp==2
        newdata{2} = log(double(Descriptor{jj})+1)-3;
      end
    else
      if useApp==1
        newdata{1} = imagedata.appearance{jj};
      elseif useApp==2
        newdata{1} = log(double(Descriptor{jj})+1)-3;
      end
    end

    OPinfS = insertdata(OPinfS, newdata, jj, category);
  end

  % Gibbs sampling: shared object model
  fprintf(1, 'Shared sampling for %d iterations...\n', numgibbs);
  trainS = intersect(traingroup, find(imagedata.classes>0));
  [OPinfS,catprobS] = doGibbs(OPinfS, numgibbs, [], trainS, [1:numimage], testiter);
  catprobS = cat(1, catprobS, catprobB);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Train Unshared Object Model
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  fprintf(1, 'Training unshared models with %d parts...\n', numclassNS(zz)); 

  % avoid memory overflow by only remembering single unshared model 
  catprobNS = [];
  for oo = 1:numobj
    % initialize unshared object-part hierarchies
    OPinfNS = objectpart(numimage,1,numclassNS(zz),alpha0/numclassNS(zz),dataprior);
    
    for jj = 1:numimage
      category = imagedata.classes(jj);

      % Insert appropriate data for each image
      if usePos
        x = imagedata.position{jj}(1,:) + 1; % add 1 for C-->Matlab coordinates
        y = imagedata.position{jj}(2,:) + 1; % add 1 for C-->Matlab coordinates
        newdata{1} = [x; y];
        if useApp==1
          newdata{2} = imagedata.appearance{jj};
        elseif useApp==2
          newdata{2} = log(double(Descriptor{jj})+1)-3;
        end
      else
        if useApp==1
          newdata{1} = imagedata.appearance{jj};
        elseif useApp==2
          newdata{1} = log(double(Descriptor{jj})+1)-3;
        end
      end
      
      OPinfNS = insertdata(OPinfNS, newdata, jj, category==oo);
    end
  
    % Gibbs sampling: unshared object model
    fprintf(1, 'Unshared sampling %d for %d iterations...\n', oo, numgibbs);
    trainNS = intersect(traingroup, find(imagedata.classes==oo));
    [OPinfNS,catprobTmp] = doGibbs(OPinfNS, numgibbs, [], trainNS, ...
      [1:numimage], testiter);
    catprobNS = cat(1, catprobNS, catprobTmp);
  end
  catprobNS = cat(1, catprobNS, catprobB);

  % Store computed probabilities for later performance analysis
  trainhist{zz}.traingroup = traingroup;
  trainhist{zz}.testgroup  = testgroup;
  trainhist{zz}.probS      = catprobS;
  trainhist{zz}.probNS     = catprobNS;

  % Save probabilities, and corresponding parts in separate file
  save(sprintf('%s_prob%.2d',savename,numclassNS(zz)), 'trainhist', ...
    'num*', 'use*', 'alpha0', 'dataprior', 'plotstyle', 'savename', 'testiter');
  save(sprintf('%s_part%.2d',savename,numclassNS(zz)), 'OPinfS');
end

disp('DONE!')
